# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
import datetime
import json

from ..scraper import ParserScraper


class GoComics(ParserScraper):
    url = 'https://www.gocomics.com/'
    imageSearch = '//section[d:class_start("ShowComicViewer_showComicViewer")]//script[@type="application/ld+json"]/text()'
    prevSearch = '//a[d:class_start("ComicNavigation_controls__button_previous__")]'
    help = 'Index format: yyyy/mm/dd'

    def __init__(self, name, path, lang=None):
        super().__init__('GoComics/' + name)
        self.session.add_throttle('www.gocomics.com', 3.0, 5.0)
        self.url = 'https://www.gocomics.com/' + path
        self.shortname = name
        if lang:
            self.lang = lang

    def imageUrlModifier(self, image_url, data):
        # we extracted a JSON object here
        self.jsondata = json.loads(image_url)
        return self.jsondata["url"]

    def namer(self, image_url, page_url):
        # We cannot use the current page URL, since even with a bounce starter,
        # the current URL doesn't contain the current date... There is
        # '//div/@data-post-url', but that only exists on comics which have
        # discussions enabled...
        datestr = self.jsondata["datePublished"]
        date = datetime.datetime.strptime(datestr, "%B %d, %Y")
        return f"{self.shortname}_{date.year}{date.month:02}{date.day:02}"

    def getIndexStripUrl(self, index):
        return f'{self.url}/{index}'

    @classmethod
    def getmodules(cls):
        return (
            # do not edit anything below since these entries are generated from
            # scripts/gocomics.py
            # START AUTOUPDATE
            cls('1AndDone', '1-and-done'),
            cls('9ChickweedLane', '9-chickweed-lane'),
            cls('9To5', '9to5'),
            cls('Aaggghhh', 'aaggghhh', 'es'),
            cls('AdamAtHome', 'adamathome'),
            cls('AdultChildren', 'adult-children'),
            cls('Agnes', 'agnes'),
            cls('AJAndMagnus', 'aj-and-magnus'),
            cls('AlGoodwynEditorialCartoons', 'algoodwyn'),
            cls('AlisHouse', 'alis-house'),
            cls('AlleyOop', 'alley-oop'),
            cls('AmandaTheGreat', 'amanda-the-great'),
            cls('Andertoons', 'andertoons'),
            cls('AndyCapp', 'andycapp'),
            cls('AngryLittleGirls', 'angry-little-girls'),
            cls('AnimalCrackers', 'animalcrackers'),
            cls('Annie', 'annie'),
            cls('AProblemLikeJamal', 'a-problem-like-jamal'),
            cls('ArloAndJanis', 'arloandjanis'),
            cls('ArtByMoga', 'artbymoga'),
            cls('AskShagg', 'askshagg'),
            cls('AtTavicat', 'tavicat'),
            cls('AuntyAcid', 'aunty-acid'),
            cls('BabyBlues', 'babyblues'),
            cls('BackInTheDay', 'backintheday'),
            cls('BackToBC', 'back-to-bc'),
            cls('Bacon', 'bacon'),
            cls('BadMachinery', 'bad-machinery'),
            cls('Baldo', 'baldo'),
            cls('BaldoEnEspanol', 'baldoespanol', 'es'),
            cls('BallardStreet', 'ballardstreet'),
            cls('BananaTriangle', 'banana-triangle'),
            cls('BarkeaterLake', 'barkeaterlake'),
            cls('BarneyAndClyde', 'barneyandclyde'),
            cls('BasicInstructions', 'basicinstructions'),
            cls('BatchRejection', 'batch-rejection'),
            cls('BC', 'bc'),
            cls('Beardo', 'beardo'),
            cls('Ben', 'ben'),
            cls('BenitinYEneas', 'muttandjeffespanol', 'es'),
            cls('BergerAndWyse', 'berger-and-wyse'),
            cls('BerkeleyMews', 'berkeley-mews'),
            cls('Betty', 'betty'),
            cls('BFGFSyndrome', 'bfgf-syndrome'),
            cls('BigNate', 'bignate'),
            cls('BigTop', 'bigtop'),
            cls('BillBramhall', 'bill-bramhall'),
            cls('BirdAndMoon', 'bird-and-moon'),
            cls('Birdbrains', 'birdbrains'),
            cls('BleekerTheRechargeableDog', 'bleeker'),
            cls('Bliss', 'bliss'),
            cls('BloomCounty', 'bloomcounty'),
            cls('BloomCountyContinues', 'bloom-county'),
            cls('BobGorrell', 'bobgorrell'),
            cls('BobTheAngryFlower', 'bob-the-angry-flower'),
            cls('BobTheSquirrel', 'bobthesquirrel'),
            cls('BoNanas', 'bonanas'),
            cls('Boomerangs', 'boomerangs'),
            cls('Bottomliners', 'bottomliners'),
            cls('BoundAndGagged', 'boundandgagged'),
            cls('Bozo', 'bozo'),
            cls('BreakingCatNews', 'breaking-cat-news'),
            cls('Brevity', 'brevity'),
            cls('BrewsterRockit', 'brewsterrockit'),
            cls('BrianMcFadden', 'brian-mcfadden'),
            cls('BroomHilda', 'broomhilda'),
            cls('Buckles', 'buckles'),
            cls('Bully', 'bully'),
            cls('Buni', 'buni'),
            cls('CalvinAndHobbes', 'calvinandhobbes'),
            cls('CalvinAndHobbesEnEspanol', 'calvinandhobbesespanol', 'es'),
            cls('CatanaComics', 'little-moments-of-love'),
            cls('Cathy', 'cathy'),
            cls('CathyCommiserations', 'cathy-commiserations'),
            cls('CatsCafe', 'cats-cafe'),
            cls('CattitudeDoggonit', 'cattitude-doggonit'),
            cls('CestLaVie', 'cestlavie'),
            cls('CheerUpEmoKid', 'cheer-up-emo-kid'),
            cls('ChipBok', 'chipbok'),
            cls('ChrisBritt', 'chrisbritt'),
            cls('ChuckDrawsThings', 'chuckdrawsthings'),
            cls('ChuckleBros', 'chucklebros'),
            cls('CitizenDog', 'citizendog'),
            cls('Claw', 'claw'),
            cls('ClayBennett', 'claybennett'),
            cls('ClayJones', 'clayjones'),
            cls('Cleats', 'cleats'),
            cls('CloseToHome', 'closetohome'),
            cls('Computoon', 'compu-toon'),
            cls('Cornered', 'cornered'),
            cls('CowAndBoyClassics', 'cowandboy'),
            cls('CowTown', 'cowtown'),
            cls('Crabgrass', 'crabgrass'),
            cls('Crankshaft', 'crankshaft'),
            cls('Crumb', 'crumb'),
            cls('CulDeSac', 'culdesac'),
            cls('Curses', 'curses'),
            cls('DaddysHome', 'daddyshome'),
            cls('DanaSummers', 'danasummers'),
            cls('DarkSideOfTheHorse', 'darksideofthehorse'),
            cls('DayByDave', 'day-by-dave'),
            cls('DeepDarkFears', 'deep-dark-fears'),
            cls('DeFlocked', 'deflocked'),
            cls('DiamondLil', 'diamondlil'),
            cls('DickTracy', 'dicktracy'),
            cls('DinosaurComics', 'dinosaur-comics'),
            cls('DogEatDoug', 'dogeatdoug'),
            cls('DogsOfCKennel', 'dogsofckennel'),
            cls('DomesticAbuse', 'domesticabuse'),
            cls('DonBrutus', 'don-brutus', 'es'),
            cls('DoodleForFood', 'doodle-for-food'),
            cls('DoodleTown', 'doodle-town'),
            cls('Doonesbury', 'doonesbury'),
            cls('Drabble', 'drabble'),
            cls('DrewSheneman', 'drewsheneman'),
            cls('EdgeCity', 'edge-city'),
            cls('Eek', 'eek'),
            cls('ElCafDePoncho', 'el-cafe-de-poncho', 'es'),
            cls('EmmyLou', 'emmy-lou'),
            cls('Endtown', 'endtown'),
            cls('EricAllie', 'eric-allie'),
            cls('Eyebeam', 'eyebeam'),
            cls('FamilyTree', 'familytree'),
            cls('Farcus', 'farcus'),
            cls('FatCats', 'fat-cats'),
            cls('FloAndFriends', 'floandfriends'),
            cls('FMinus', 'fminus'),
            cls('FoolishMortals', 'foolish-mortals'),
            cls('ForBetterOrForWorse', 'forbetterorforworse'),
            cls('ForHeavensSake', 'forheavenssake'),
            cls('FourEyes', 'four-eyes'),
            cls('FowlLanguage', 'fowl-language'),
            cls('FoxTrot', 'foxtrot'),
            cls('FoxTrotClassics', 'foxtrotclassics'),
            cls('FoxTrotEnEspanol', 'foxtrotespanol', 'es'),
            cls('Francis', 'francis'),
            cls('FrankAndErnest', 'frank-and-ernest'),
            cls('Frazz', 'frazz'),
            cls('FredBasset', 'fredbasset'),
            cls('FredBassetEnEspanol', 'fredbassetespanol', 'es'),
            cls('FreeRange', 'freerange'),
            cls('FreshlySqueezed', 'freshlysqueezed'),
            cls('FrogApplause', 'frogapplause'),
            cls('FunkyWinkerbean', 'funky-winkerbean'),
            cls('FurBabies', 'furbabies'),
            cls('Garfield', 'garfield'),
            cls('GarfieldEnEspanol', 'garfieldespanol', 'es'),
            cls('GaryMarkstein', 'garymarkstein'),
            cls('GaryVarvel', 'garyvarvel'),
            cls('GasolineAlley', 'gasolinealley'),
            cls('Gaturro', 'gaturro', 'es'),
            cls('Geech', 'geech'),
            cls('GetALife', 'getalife'),
            cls('GetFuzzy', 'getfuzzy'),
            cls('Gil', 'gil'),
            cls('GilThorp', 'gilthorp'),
            cls('GingerMeggs', 'gingermeggs'),
            cls('GingerMeggsEnEspanol', 'gingermeggs-espanol', 'es'),
            cls('GlasbergenCartoons', 'glasbergen-cartoons'),
            cls('Globetrotter', 'globetrotter'),
            cls('GManWebcomics', 'g-man-webcomics'),
            cls('Goats', 'goats'),
            cls('GrandAvenue', 'grand-avenue'),
            cls('GrayMatters', 'gray-matters'),
            cls('GreenHumour', 'green-humour'),
            cls('HaircutPractice', 'haircut-practice'),
            cls('HalfFull', 'half-full'),
            cls('Harley', 'harley'),
            cls('HeartOfTheCity', 'heartofthecity'),
            cls('Heathcliff', 'heathcliff'),
            cls('HeathcliffEnEspanol', 'heathcliffespanol', 'es'),
            cls('HenryPayne', 'henrypayne'),
            cls('HerbAndJamaal', 'herbandjamaal'),
            cls('Herman', 'herman'),
            cls('HomeAndAway', 'homeandaway'),
            cls('HomeFree', 'homefree'),
            cls('HotComicsForCoolPeople', 'hot-comics-for-cool-people'),
            cls('HutchOwen', 'hutch-owen'),
            cls('ImagineThis', 'imaginethis'),
            cls('ImogenQuest', 'imogen-quest'),
            cls('InkPen', 'inkpen'),
            cls('InSecurity', 'in-security'),
            cls('InTheBleachers', 'inthebleachers'),
            cls('InTheSticks', 'inthesticks'),
            cls('InvisibleBread', 'invisible-bread'),
            cls('ItsAllAboutYou', 'itsallaboutyou'),
            cls('JackOhman', 'jackohman'),
            cls('JakeLikesOnions', 'jake-likes-onions'),
            cls('JanesWorld', 'janesworld'),
            cls('JeffDanziger', 'jeffdanziger'),
            cls('JeffStahler', 'jeffstahler'),
            cls('JenSorensen', 'jen-sorensen'),
            cls('JerryKingComics', 'jerry-king-comics'),
            cls('JimBentonCartoons', 'jim-benton-cartoons'),
            cls('JimMorin', 'jimmorin'),
            cls('JoeHeller', 'joe-heller'),
            cls('JoelPett', 'joelpett'),
            cls('JoeyWeatherford', 'joey-weatherford'),
            cls('JohnDeering', 'johndeering'),
            cls('JumpStart', 'jumpstart'),
            cls('JunkDrawer', 'junk-drawer'),
            cls('JustoYFranco', 'justo-y-franco', 'es'),
            cls('KevinKallaugher', 'kal'),
            cls('KevinNecessaryEditorialCartoons', 'kevin-necessary-editorial-cartoons'),
            cls('KidBeowulf', 'kid-beowulf'),
            cls('KitchenCapers', 'kitchen-capers'),
            cls('Kliban', 'kliban'),
            cls('KlibansCats', 'klibans-cats'),
            cls('LaCucaracha', 'lacucaracha'),
            cls('LaCucarachaEnEspanol', 'la-cucaracha-en-espanol', 'es'),
            cls('LaloAlcaraz', 'laloalcaraz'),
            cls('LardsWorldPeaceTips', 'lards-world-peace-tips'),
            cls('LasHermanasStone', 'stonesoup_espanol', 'es'),
            cls('LastKiss', 'lastkiss'),
            cls('LaughingRedheadComics', 'laughing-redhead-comics'),
            cls('LayLines', 'lay-lines'),
            cls('LearnToSpeakCat', 'learn-to-speak-cat'),
            cls('LibertyMeadows', 'libertymeadows'),
            cls('LifeOnEarth', 'life-on-earth'),
            cls('LilAbner', 'lil-abner'),
            cls('Lio', 'lio'),
            cls('LioEnEspanol', 'lioespanol', 'es'),
            cls('LisaBenson', 'lisabenson'),
            cls('LittleDogLost', 'littledoglost'),
            cls('LittleFriedChickenAndSushi', 'little-fried-chicken-and-sushi'),
            cls('LittleNemo', 'little-nemo'),
            cls('LizClimoCartoons', 'liz-climo-cartoons'),
            cls('Lola', 'lola'),
            cls('LolaEnEspanol', 'lola-en-espanol', 'es'),
            cls('LongStoryShort', 'long-story-short'),
            cls('LooksGoodOnPaper', 'looks-good-on-paper'),
            cls('LooseParts', 'looseparts'),
            cls('LosOsorios', 'los-osorios', 'es'),
            cls('LostSheep', 'lostsheep'),
            cls('Luann', 'luann'),
            cls('LuannAgainn', 'luann-againn'),
            cls('LuannEnEspanol', 'luannspanish', 'es'),
            cls('LuckyCow', 'luckycow'),
            cls('LugNuts', 'lug-nuts'),
            cls('Lunarbaboon', 'lunarbaboon'),
            cls('M2Bulls', 'm2bulls'),
            cls('Maintaining', 'maintaining'),
            cls('MannequinOnTheMoon', 'mannequin-on-the-moon'),
            cls('MariasDay', 'marias-day'),
            cls('Marmaduke', 'marmaduke'),
            cls('MarshallRamsey', 'marshallramsey'),
            cls('MattBors', 'matt-bors'),
            cls('MattDavies', 'mattdavies'),
            cls('MattWuerker', 'mattwuerker'),
            cls('MediumLarge', 'medium-large'),
            cls('MessycowComics', 'messy-cow'),
            cls('MexikidStories', 'mexikid-stories'),
            cls('MichaelRamirez', 'michaelramirez'),
            cls('MikeBeckom', 'mike-beckom'),
            cls('MikeDuJour', 'mike-du-jour'),
            cls('MikeLester', 'mike-lester'),
            cls('MikeLuckovich', 'mikeluckovich'),
            cls('MissPeach', 'miss-peach'),
            cls('ModeratelyConfused', 'moderately-confused'),
            cls('MollyAndTheBear', 'molly-and-the-bear'),
            cls('Momma', 'momma'),
            cls('Monty', 'monty'),
            cls('MontyDiaros', 'monty-diaros', 'es'),
            cls('MotherGooseAndGrimm', 'mother-goose-and-grimm'),
            cls('MotleyClassics', 'motley-classics'),
            cls('MrLowe', 'mr-lowe'),
            cls('MuttAndJeff', 'muttandjeff'),
            cls('MyDadIsDracula', 'my-dad-is-dracula'),
            cls('MythTickle', 'mythtickle'),
            cls('Nancy', 'nancy'),
            cls('NancyClassics', 'nancy-classics'),
            cls('NateElGrande', 'nate-el-grande', 'es'),
            cls('NestHeads', 'nestheads'),
            cls('NEUROTICA', 'neurotica'),
            cls('NewAdventuresOfQueenVictoria', 'thenewadventuresofqueenvictoria'),
            cls('NextDoorNeighbors', 'next-door-neighbors'),
            cls('NickAnderson', 'nickanderson'),
            cls('NickAndZuzu', 'nick-and-zuzu'),
            cls('NonSequitur', 'nonsequitur'),
            cls('NothingIsNotSomething', 'nothing-is-not-something'),
            cls('NotInventedHere', 'not-invented-here'),
            cls('NowRecharging', 'now-recharging'),
            cls('OffTheMark', 'offthemark'),
            cls('OhBrother', 'oh-brother'),
            cls('OllieAndQuentin', 'ollie-and-quentin'),
            cls('OnAClaireDay', 'onaclaireday'),
            cls('OneBigHappy', 'onebighappy'),
            cls('OrdinaryBill', 'ordinary-bill'),
            cls('OriginsOfTheSundayComics', 'origins-of-the-sunday-comics'),
            cls('OurSuperAdventure', 'our-super-adventure'),
            cls('Outland', 'outland'),
            cls('OutOfTheGenePool', 'outofthegenepool'),
            cls('Overboard', 'overboard'),
            cls('OverboardEnEspanol', 'overboardespanol', 'es'),
            cls('OverTheHedge', 'overthehedge'),
            cls('OzyAndMillie', 'ozy-and-millie'),
            cls('PatOliphant', 'patoliphant'),
            cls('Peanuts', 'peanuts'),
            cls('PeanutsBegins', 'peanuts-begins'),
            cls('PeanutsEnEspanol', 'peanuts-espanol', 'es'),
            cls('PearlsBeforeSwine', 'pearlsbeforeswine'),
            cls('PedroXMolina', 'pedroxmolina'),
            cls('Periquita', 'periquita', 'es'),
            cls('PerlasParaLosCerdos', 'perlas-para-los-cerdos', 'es'),
            cls('PerryBibleFellowship', 'perry-bible-fellowship'),
            cls('PhilHands', 'phil-hands'),
            cls('PhoebeAndHerUnicorn', 'phoebe-and-her-unicorn'),
            cls('Pibgorn', 'pibgorn'),
            cls('PibgornSketches', 'pibgornsketches'),
            cls('Pickles', 'pickles'),
            cls('PleaseListenToMe', 'please-listen-to-me'),
            cls('Pluggers', 'pluggers'),
            cls('PoochCafe', 'poochcafe'),
            cls('Poorcraft', 'poorcraft'),
            cls('PoorlyDrawnLines', 'poorly-drawn-lines'),
            cls('PotShots', 'pot-shots'),
            cls('PreTeena', 'preteena'),
            cls('PricklyCity', 'pricklycity'),
            cls('QuestionableQuotebook', 'questionable-quotebook'),
            cls('RabbitsAgainstMagic', 'rabbitsagainstmagic'),
            cls('RaisingDuncan', 'raising-duncan'),
            cls('RandolphItch2Am', 'randolphitch'),
            cls('RealityCheck', 'realitycheck'),
            cls('RealLifeAdventures', 'reallifeadventures'),
            cls('RebeccaHendin', 'rebecca-hendin'),
            cls('RedAndRover', 'redandrover'),
            cls('RedMeat', 'redmeat'),
            cls('RichardsPoorAlmanac', 'richards-poor-almanac'),
            cls('RipHaywire', 'riphaywire'),
            cls('RipleysAunqueUstedNoLoCrea', 'ripleys-en-espanol', 'es'),
            cls('RipleysBelieveItOrNot', 'ripleysbelieveitornot'),
            cls('RobbieAndBobby', 'robbie-and-bobby'),
            cls('RobertAriail', 'robert-ariail'),
            cls('RobRogers', 'robrogers'),
            cls('RoseIsRose', 'roseisrose'),
            cls('Rubes', 'rubes'),
            cls('SarahsScribbles', 'sarahs-scribbles'),
            cls('SaturdayMorningBreakfastCereal', 'saturday-morning-breakfast-cereal'),
            cls('SavageChickens', 'savage-chickens'),
            cls('ScaryGary', 'scarygary'),
            cls('ScenesFromAMultiverse', 'scenes-from-a-multiverse'),
            cls('ScottStantis', 'scottstantis'),
            cls('ShenComix', 'shen-comix'),
            cls('ShermansLagoon', 'shermanslagoon'),
            cls('ShirleyAndSonClassics', 'shirley-and-son-classics'),
            cls('Shoe', 'shoe'),
            cls('SketchsharkComics', 'sketchshark-comics'),
            cls('SkinHorse', 'skinhorse'),
            cls('Skippy', 'skippy'),
            cls('SmallPotatoes', 'small-potatoes'),
            cls('SnowSez', 'snow-sez'),
            cls('SpeedBump', 'speedbump'),
            cls('SpiritOfTheStaircase', 'spirit-of-the-staircase'),
            cls('SpotTheFrog', 'spot-the-frog'),
            cls('SteveBenson', 'stevebenson'),
            cls('SteveBreen', 'stevebreen'),
            cls('SteveKelley', 'stevekelley'),
            cls('StickyComics', 'sticky-comics'),
            cls('StoneSoup', 'stonesoup'),
            cls('StrangeBrew', 'strangebrew'),
            cls('StudioJantze', 'studio-jantze'),
            cls('SunnyStreet', 'sunny-street'),
            cls('SuperFunPakComix', 'super-fun-pak-comix'),
            cls('SwanEaters', 'swan-eaters'),
            cls('SweetAndSourPork', 'sweet-and-sour-pork'),
            cls('Sylvia', 'sylvia'),
            cls('TankMcNamara', 'tankmcnamara'),
            cls('Tarzan', 'tarzan'),
            cls('TarzanEnEspanol', 'tarzan-en-espanol', 'es'),
            cls('TedRall', 'ted-rall'),
            cls('TenCats', 'ten-cats'),
            cls('Tex', 'tex'),
            cls('TextsFromMittens', 'texts-from-mittens'),
            cls('Thatababy', 'thatababy'),
            cls('ThatIsPriceless', 'that-is-priceless'),
            cls('ThatNewCarlSmell', 'that-new-carl-smell'),
            cls('TheAcademiaWaltz', 'the-academia-waltz'),
            cls('TheAdventuresOfBusinessCat', 'the-adventures-of-business-cat'),
            cls('TheAncients', 'the-ancients'),
            cls('TheArgyleSweater', 'theargylesweater'),
            cls('TheAwkwardYeti', 'the-awkward-yeti'),
            cls('TheBarn', 'thebarn'),
            cls('TheBigPicture', 'thebigpicture'),
            cls('TheBoondocks', 'boondocks'),
            cls('TheBornLoser', 'the-born-loser'),
            cls('TheBuckets', 'thebuckets'),
            cls('TheCity', 'thecity'),
            cls('TheComicStripThatHasAFinaleEveryDay', 'the-comic-strip-that-has-a-finale-every-day'),
            cls('TheDailyDrawing', 'the-daily-drawing'),
            cls('TheDinetteSet', 'dinetteset'),
            cls('TheDoozies', 'thedoozies'),
            cls('TheDuplex', 'duplex'),
            cls('TheElderberries', 'theelderberries'),
            cls('TheFlyingMcCoys', 'theflyingmccoys'),
            cls('TheFuscoBrothers', 'thefuscobrothers'),
            cls('TheGrizzwells', 'thegrizzwells'),
            cls('TheHumbleStumble', 'humble-stumble'),
            cls('TheKChronicles', 'thekchronicles'),
            cls('TheKnightLife', 'theknightlife'),
            cls('TheLockhorns', 'lockhorns'),
            cls('TheMartianConfederacy', 'the-martian-confederacy'),
            cls('TheMeaningOfLila', 'meaningoflila'),
            cls('TheMiddleAge', 'the-middle-age'),
            cls('TheMiddletons', 'themiddletons'),
            cls('TheNormClassics', 'thenorm'),
            cls('TheOtherCoast', 'theothercoast'),
            cls('TheUpsideDownWorldOfGustaveVerbeek', 'upside-down-world-of-gustave-verbeek'),
            cls('TheWanderingMelon', 'the-wandering-melon'),
            cls('TheWorriedWell', 'the-worried-well'),
            cls('think', 'think'),
            cls('ThinLines', 'thinlines'),
            cls('TimCampbell', 'tim-campbell'),
            cls('TinySepuku', 'tinysepuku'),
            cls('TodaysSzep', 'todays-szep'),
            cls('TomTheDancingBug', 'tomthedancingbug'),
            cls('TomToles', 'tomtoles'),
            cls('TooMuchCoffeeMan', 'toomuchcoffeeman'),
            cls('Trucutu', 'trucutu', 'es'),
            cls('Tutelandia', 'tutelandia', 'es'),
            cls('TwoPartyOpera', 'two-party-opera'),
            cls('UFO', 'ufo'),
            cls('UnderpantsAndOverbites', 'underpants-and-overbites'),
            cls('UnderstandingChaos', 'understanding-chaos'),
            cls('UnstrangePhenomena', 'unstrange-phenomena'),
            cls('ViewsAfrica', 'viewsafrica'),
            cls('ViewsAmerica', 'viewsamerica'),
            cls('ViewsAsia', 'viewsasia'),
            cls('ViewsBusiness', 'viewsbusiness'),
            cls('ViewsEurope', 'viewseurope'),
            cls('ViewsLatinAmerica', 'viewslatinamerica'),
            cls('ViewsMidEast', 'viewsmideast'),
            cls('ViewsOfTheWorld', 'viewsoftheworld'),
            cls('ViiviAndWagner', 'viivi-and-wagner'),
            cls('WallaceTheBrave', 'wallace-the-brave'),
            cls('WaltHandelsman', 'walthandelsman'),
            cls('Warped', 'warped'),
            cls('WatchYourHead', 'watchyourhead'),
            cls('Wawawiwa', 'wawawiwa'),
            cls('WaynoVision', 'waynovision'),
            cls('WeePals', 'weepals'),
            cls('WideOpen', 'wide-open'),
            cls('WinLoseDrew', 'drewlitton'),
            cls('WizardOfId', 'wizardofid'),
            cls('WizardOfIdClassics', 'wizard-of-id-classics'),
            cls('WizardOfIdEnEspanol', 'wizardofidespanol', 'es'),
            cls('Wondermark', 'wondermark'),
            cls('WorkingDaze', 'working-daze'),
            cls('WorkingItOut', 'workingitout'),
            cls('WorryLines', 'worry-lines'),
            cls('WrongHands', 'wrong-hands'),
            cls('WTDuck', 'wtduck'),
            cls('WuMo', 'wumo'),
            cls('WuMoEnEspanol', 'wumoespanol', 'es'),
            cls('Yaffle', 'yaffle'),
            cls('YeahItsChill', 'yeah-its-chill'),
            cls('YesImHotInThis', 'yesimhotinthis'),
            cls('ZackHill', 'zackhill'),
            cls('ZenPencils', 'zen-pencils'),
            cls('Ziggy', 'ziggy'),
            cls('ZiggyEnEspanol', 'ziggyespanol', 'es'),
            # END AUTOUPDATE
        )



# SPDX-License-Identifier: MIT
# Copyright (C) 2013-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
from pathlib import Path

import pytest

from dosagelib.scraper import scrapers


class TestScraper(object):
    """Test scraper module functions."""

    def test_get_scrapers(self):
        for scraperobj in scrapers.all():
            scraperobj.indexes = ["bla"]
            assert scraperobj.url, "missing url in %s" % scraperobj.name

    def test_find_scrapers_single(self):
        assert scrapers.find("xkcd")

    def test_find_scrapers_multi(self):
        with pytest.raises(ValueError, match='multiple comics found'):
            scrapers.find("a")

    def test_find_scrapers_error(self):
        with pytest.raises(ValueError, match='empty comic name'):
            scrapers.find('')

    def test_user_dir(self):
        oldlen = len(scrapers.all())
        scrapers.adddir(Path(__file__).parent / 'mocks' / 'extra')
        assert len(scrapers.all()) == oldlen + 1
        assert scrapers.find('AnotherDummyTestScraper')



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
"""
Define basic configuration data like version or application name.
"""
from . import AppName, __version__

App = f'{AppName} {__version__}'

Maintainer = 'Tobias Gruetzmacher'
MaintainerEmail = 'tobias-dosage@23.gs'
Url = 'https://dosage.rocks/'
SupportUrl = 'https://github.com/webcomics/dosage/issues'
UserAgent = f"Mozilla/5.0 (compatible; {AppName}/{__version__}; +{Url})"
Copyright = """Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
Copyright (C) 2012-2014 Bastian Kleineidam
Copyright (C) 2015-2025 Tobias Gruetzmacher
Copyright (C) 2019-2020 Daniel Ring
"""
Freeware = f"""{AppName} comes with ABSOLUTELY NO WARRANTY!
This is free software, and you are welcome to redistribute it
under certain conditions. Look at the file `COPYING' within this
distribution."""
VoteUrl = "https://buildbox.23.gs/count/"



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
# Copyright (C) 2019-2020 Daniel Ring
from typing import Sequence, Union

from ..scraper import ParserScraper

# Common base classes for comics with the same structure (same hosting
# software, for example) go here. Since those are shared by many modules,
# please don't use lists of expression, as that makes it hard to track which
# expression is for which comics.
__all__ = (
    'ComicControlScraper',
    'WordPressNavi',
    'WordPressNaviIn',
    'WordPressScraper',
    'WordPressSpliced',
    'WordPressWebcomic',
)


class ComicControlScraper(ParserScraper):
    imageSearch: Union[Sequence[str], str] = '//img[@id="cc-comic"]'
    prevSearch = '//a[@rel="prev"]'
    nextSearch = '//a[@rel="next"]'
    latestSearch = '//a[@rel="last"]'


class WordPressScraper(ParserScraper):
    imageSearch = '//div[@id="comic"]//img'
    prevSearch = '//a[d:class("comic-nav-previous")]'
    nextSearch = '//a[d:class("comic-nav-next")]'
    latestSearch = '//a[d:class("comic-nav-last")]'


class WordPressSpliced(ParserScraper):
    imageSearch = '//div[@id="one-comic-option"]//img'
    prevSearch = '//a[d:class("previous-comic")]'


class WordPressNavi(WordPressScraper):
    prevSearch = '//a[d:class("navi-prev")]'


class WordPressNaviIn(WordPressScraper):
    prevSearch = '//a[d:class("navi-prev-in")]'


class WordPressWebcomic(ParserScraper):
    imageSearch = '//div[d:class("webcomic-image")]//img'
    prevSearch = '//a[d:class("previous-webcomic-link")]'
    nextSearch = '///a[d:class("next-webcomic-link")]'
    latestSearch = '//a[d:class("last-webcomic-link")]'



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2020 Tobias Gruetzmacher
# Copyright (C) 2019-2020 Daniel Ring
from .common import WordPressScraper
from ..helpers import indirectStarter


class WebcomicFactory(WordPressScraper):
    starter = indirectStarter

    def __init__(self, name, url):
        super(WebcomicFactory, self).__init__(name)
        self.url = url
        self.firstStripUrl = url

    @classmethod
    def getmodules(cls):
        return (
            # do not edit anything below since these entries are generated from
            # scripts/webcomicfactory.py
            # START AUTOUPDATE
            cls('AsTheMayoTurns',
                'http://www.thewebcomicfactory.com/comic/as-the-mayo-turns/'),
            cls('ComicBookMafia',
                'http://www.thewebcomicfactory.com/comic/comic-book-mafia/'),
            cls('Dealers',
                'http://www.thewebcomicfactory.com/comic/dealers-1-1998-was-the-year/'),
            cls('DigitalHobo',
                'http://www.thewebcomicfactory.com/comic/digital-hobo-1-its-a-living-kinda/'),
            cls('ECoastVsWCoast',
                'http://www.thewebcomicfactory.com/comic/east-coast-vs-west-coast-greetings-from-the-coasts/'),
            cls('GunCulture',
                'http://www.thewebcomicfactory.com/comic/gun-culture/'),
            cls('IHateMyKids',
                'http://www.thewebcomicfactory.com/comic/i-hate-my-kids/'),
            cls('InARelationship',
                'http://www.thewebcomicfactory.com/comic/in-a-relationship-3/'),
            cls('IntergalacticMedicalDoctor',
                'http://www.thewebcomicfactory.com/comic/intergalactic-medical-doctor/'),
            cls('JSchoolgirlsInLove',
                'http://www.thewebcomicfactory.com/comic/japanese-schoolgirls-in-love-1/'),
            cls('KingdomOfTheDwarves',
                'http://www.thewebcomicfactory.com/comic/kingdom-of-the-dwarves/'),
            cls('LesterCrenshawIsDead',
                'http://www.thewebcomicfactory.com/comic/lester-crenshaw-is-dead/'),
            cls('Millennials',
                'http://www.thewebcomicfactory.com/comic/millennials/'),
            cls('MiserableComedians',
                'http://www.thewebcomicfactory.com/comic/miserable-comedians-1-funny-because-its-sad/'),
            cls('OldeTymeGamer',
                'http://www.thewebcomicfactory.com/comic/olde-tyme-gamer-playing-injured/'),
            cls('PinJunkies',
                'http://www.thewebcomicfactory.com/comic/pin-junkies/'),
            cls('PostApocalypticNick',
                'http://www.thewebcomicfactory.com/comic/post-apocalyptic-nick/'),
            cls('RealTalk',
                'http://www.thewebcomicfactory.com/comic/real-talk-people-who-cut-in-line/'),
            cls('SoManyNightmares',
                'http://www.thewebcomicfactory.com/comic/so-many-nightmares-freedom-nightmare/'),
            cls('SportsGuys',
                'http://www.thewebcomicfactory.com/comic/sports-guys/'),
            cls('TalesOfPizza',
                'http://www.thewebcomicfactory.com/comic/tales-of-pizza-bad-tipper/'),
            cls('TAndA',
                'http://www.thewebcomicfactory.com/comic/the-webcomic-factory-premiere-t-and-a/'),
            cls('TheAntiwarComic',
                'http://www.thewebcomicfactory.com/comic/the-antiwar-comic-the-party/'),
            cls('TheGentlemensClub',
                'http://www.thewebcomicfactory.com/comic/the-gentlemens-club/'),
            cls('TheHorrorOfColony6',
                'http://www.thewebcomicfactory.com/comic/the-horror-of-colony-6-page-1/'),
            cls('TheKingsOfViralVideo',
                'http://www.thewebcomicfactory.com/comic/the-kings-of-viral-video-premiere/'),
            cls('TheSharonAndTonyExperiment',
                'http://www.thewebcomicfactory.com/comic/the-sharon-and-tony-experiment/'),
            cls('TonyDestructo',
                'http://www.thewebcomicfactory.com/comic/tony-destructo/'),
            cls('WeirdBikerTales',
                'http://www.thewebcomicfactory.com/comic/weird-biker-tales-the-last-outlaw/'),
            cls('WillysSpaceDive',
                'http://www.thewebcomicfactory.com/comic/willys-space-dive/'),
            # END AUTOUPDATE
        )



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
from .. import helpers, scraper, util


class CloneManga(scraper.ParserScraper):
    baseUrl = 'http://manga.clone-army.org'
    imageSearch = '//div[d:class("subsectionContainer")]//img'
    prevSearch = '//a[span[text()="<<"]]'
    latestSearch = '//a[span[text()=">|"]]'
    starter = helpers.indirectStarter
    help = 'Index format: n'

    def __init__(self, name, shortName, endOfLife=False) -> None:
        super().__init__('CloneManga/' + name)
        self.stripUrl = '%s/viewer.php?page=%%s&lang=&series=%s&HUDoff=' % (
            self.baseUrl, shortName)
        self.url = self.stripUrl % '1'
        self.endOfLife = endOfLife

    def namer(self, image_url, page_url):
        origname = util.urlpathsplit(image_url)[-1]
        params = util.getQueryParams(page_url)
        if 'page' in params:
            return '{:03}_{}'.format(int(params['page'][0]), origname)
        else:
            return origname

    @classmethod
    def getmodules(cls):
        return (
            cls('ACaptainsWorries', 'captains_worries'),
            cls('AHimehornsDailyLife', 'himehorn'),
            cls('AprilAndMay', 'anm', endOfLife=True),
            cls('DollAndMaker', 'maria_doll', endOfLife=True),
            cls('Kanami', 'kanami', endOfLife=True),
            cls('MomokaCorner', 'momoka', endOfLife=True),
            cls('MyShutInVampirePrincess', 'snax'),
            cls('NanasEverydayLife', 'nana', endOfLife=True),
            cls('NNN', 'nnn', endOfLife=True),
            cls('PaperEleven', 'pxi', endOfLife=True),
            cls('PennyTribute', 'penny', endOfLife=True),
            cls('Tomoyo42sRoom', 't42r'),
        )



#!/usr/bin/env python3
# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
"""
Script to get a list of creators.com comics and save the info in a JSON file
for further processing.
"""

import scriptutil


class CreatorsUpdater(scriptutil.ComicListUpdater):
    dup_templates = ('GoComics/%s',)

    # names of comics to exclude
    excluded_comics = (
        # no images
        'Doodles',
    )

    def handle_url(self, url):
        """Parse one listing page."""
        data = self.get_url(url)

        for comicdiv in self.xpath(data, '//ul[d:class("all-test")]/li'):
            comiclink = self.xpath(comicdiv, './a')[0]
            comicurl = comiclink.attrib['href']
            name = self.xpath(comicdiv, './/p/strong')[0].text

            self.add_comic(name, comicurl.rsplit('/', 1)[1])

    def collect_results(self):
        """Parse all search result pages."""
        self.handle_url('https://www.creators.com/categories/comics/all')
        self.handle_url('https://www.creators.com/categories/cartoons/all')

    def get_entry(self, name, path):
        langopt = ", 'es'" if name.lower().endswith('spanish') else ''
        return u"cls('%s', '%s'%s)," % (name, path, langopt)


if __name__ == '__main__':
    CreatorsUpdater(__file__).run()



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
from ..scraper import ParserScraper
from .common import ComicControlScraper


class NuklearPower(ParserScraper):
    prevSearch = '//a[@rel="prev"]'
    imageSearch = '//div[@id="comic"]/img'

    def __init__(self, name, path):
        super().__init__('NuklearPower/' + name)
        self.url = 'http://www.nuklearpower.com/' + path + '/'

    @classmethod
    def getmodules(cls):
        return (
            cls('8BitTheater', '8-bit-theater'),
            cls('HowIKilledYourMaster', 'hikym'),
            cls('TheDreadful', 'dreadful'),
            cls('Warbot', 'warbot'),
        )

class NPAtomicRobo(ComicControlScraper):
    name = 'NuklearPower/AtomicRobo'
    url = 'http://www.atomic-robo.com/'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
import re

import pytest
import responses

import httpmocks
from dosagelib import cmd, scraper

MDAPI = 'https://api.mangadex.org/'


@pytest.fixture
def run(tmpdir):
    def _runner(*options, expected: int = 0):
        """'Fake' run dosage with given options."""
        assert cmd.main(("--allow-multiple", '--basepath', str(tmpdir)) +
            options) == expected
    return _runner


@pytest.mark.usefixtures('_nosleep', '_noappdirs')
class TestModules:
    """Test that specific comic modules work correctly."""

    @responses.activate
    def test_turnoff(self, run):
        httpmocks.page('https://turnoff.us/', 'turnoff-home')
        httpmocks.page('https://turnoff.us/geek/dev-interviews/',
            'turnoff-dev-interviews')
        httpmocks.page('https://turnoff.us/geek/git-submodules/',
            'turnoff-git-submodules')

        httpmocks.png(re.compile(r'https://turnoff\.us/image/en/.*\.png'))

        run('--numstrips', '2', 'turnoff')

    @responses.activate
    def test_gocomics_index(self, capfd, run):
        httpmocks.page('https://www.gocomics.com/calvinandhobbesespanol',
            'gocomics-root')
        httpmocks.page(re.compile('.*espanol/2020/03/25'), 'gocomics-page')
        httpmocks.page(re.compile('.*espanol/2012/07/22'), 'gocomics-page')

        httpmocks.png(re.compile(r'https://featureassets\..*'))

        run('CalvinAndHobbesEnEspanol')
        run('CalvinAndHobbesEnEspanol:2012/07/22')

        out = capfd.readouterr().out
        assert 'ERROR' not in out

    @responses.activate
    def test_unsounded(self, capfd, run):
        httpmocks.page('https://www.casualvillain.com/Unsounded/comic+index/',
            'unsounded-root')
        httpmocks.page('https://www.casualvillain.com/Unsounded/comic/ch17/ch17_92.html',
            'unsounded-17-92')
        httpmocks.page('https://www.casualvillain.com/Unsounded/comic/ch17/ch17_137.html',
            'unsounded-17-137')
        httpmocks.jpeg(re.compile(r'.*/pageart/ch\d+_\d+.jpg'))

        run('Unsounded')
        run('Unsounded:17-92')

        out = capfd.readouterr().out
        assert 'ERROR' not in out

    @responses.activate
    @pytest.mark.skip(reason="SoloLeveling was removed, so we have no way to test this...")
    def test_sololeveling_geoblock(self):
        from dosagelib.plugins.s import SoloLeveling

        responses.add(responses.GET, 'https://w3.sololeveling.net/',
            '<span>1020</span>', status=403)

        with pytest.raises(scraper.GeoblockedException):
            next(SoloLeveling.getmodules()[0].getStrips(1))

    @responses.activate
    def test_deathbulge(self, capfd, run):
        assert isinstance(scraper.scrapers.find("Deathbulge"), scraper.BasicScraper)
        httpmocks.json('https://www.deathbulge.com/api/comics', 'deathbulge-root')
        httpmocks.json('https://www.deathbulge.com/api/comics/435', 'deathbulge-435')
        httpmocks.json('https://www.deathbulge.com/api/comics/434', 'deathbulge-434')
        httpmocks.jpeg(re.compile(r'.*/images/comics/\d+.jpg'))

        run('-v', '--numstrips', '2', 'Deathbulge')

        out = capfd.readouterr().out
        assert 'ERROR' not in out

    @responses.activate
    def test_mangadex(self, capfd, run):
        httpmocks.json(MDAPI + 'manga/1b2fddf9-1385-4f3c-b37a-cf86a9428b1a', 'mangadex-isr')
        httpmocks.json(re.compile(MDAPI + '.*/feed'), 'mangadex-isr-feed')
        httpmocks.json(re.compile(MDAPI + 'chapter/'), 'mangadex-isr-chapter')
        httpmocks.json(re.compile(MDAPI + 'at-home/server/'), 'mangadex-server-at-home')
        httpmocks.png(re.compile(r'.+/data/.+.png'))

        run('-v', '--numstrips', '2', '--adult', 'MangaDex/InterspeciesReviewers')

        out = capfd.readouterr().out
        assert 'ERROR' not in out

    @responses.activate
    def test_mangadex_gone(self, capfd, run):
        httpmocks.json(MDAPI + 'manga/5c06ae70-b5cf-431a-bcd5-262a411de527', 'mangadex-aot')
        httpmocks.json(re.compile(MDAPI + '.*/feed'), 'mangadex-aot-feed')

        run('-v', 'MangaDex/DragonDrive', expected=1)

        captured = capfd.readouterr()
        assert 'ERROR: Comic' in captured.out



#!/usr/bin/env python3
# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
"""
Script to get ComicFury comics and save the info in a JSON file for further
processing.
"""

import sys
from urllib.parse import urlsplit

import scriptutil


class ComicFuryUpdater(scriptutil.ComicListUpdater):
    # Absolute minumum number of pages a comic may have (restrict search space)
    MIN_COMICS = 90

    dup_templates = ('ComicSherpa/%s', 'Creators/%s', 'GoComics/%s',
                     'KeenSpot/%s', 'Arcamax/%s')

    langmap = {
        'german': 'de',
        'spanish': 'es',
        'italian': 'it',
        'japanese': 'ja',
        'french': 'fr',
        'portuguese': 'pt',
    }

    # names of comics to exclude
    excluded_comics = (
        # unsuitable navigation
        "AlfdisAndGunnora",
        "AnAmericanNerdInAnimatedTokyo",
        "AngryAlien",
        "BoozerAndStoner",
        "Bonejangles",
        "ConradStory",
        "Crossing",
        "ChristianHumberReloaded",
        "CorkAndBlotto",
        "Democomix",
        "ErraticBeatComics",
        "EnergyWielders",
        "EvilBearorg",
        "Fiascos",
        "FateOfTheBlueStar",
        "FPK",
        "Fanartgyle",
        "FrigginRandom",
        "GoodbyeKitty",
        "GoodSirICannotDraw",
        "HighlyExperiMental",
        "IfAndCanBeFlowers",
        "JournalismStory",
        "JohnsonSuperior",
        "Keel",
        "JudgeDredBasset",
        "LomeathAndHuilii",
        "MNPB",
        "LucidsDream",
        "MadDog",
        "Minebreakers",
        "MoonlightValley",
        "MyImmortalFool",
        "NATO",
        "NothingFits",
        "OptimisticFishermenAndPessimisticFishermen",
        "Old2G",
        "NothingFitsArtBlog",
        "OutToLunchTheStingRayWhoreStory",
        "Pandemonium",
        "Pewfell",
        "ProjectX",
        "Ratantia",
        "RealLifeTrips",
        "Sandgate",
        "Secondpuberty",
        "Seconds",
        "SlightlyEccentricOrigins",
        "StardustTheCat",
        "StrangerThanFiction",
        "TalamakGreatAdventure",
        "TheBattalion",
        "TheBends",
        "TheDailyProblem",
        "TheMansionOfE",
        "ThePainter",
        "TheSeekers",
        "TheTrialsOfKlahadOfTheAbyss",
        "TheStickmen",
        "ThornsInOurSide",
        "TopHeavyVeryBustyPinUpsForAdults",
        "USBUnlimitedSimulatedBody",
        "TylerHumanRecycler",
        "UAF",
        "WhenPigsFly",
        "YeOldeLegotimeTheatre",

        # no content
        "Angst",
        "TheDevonLegacyPrologue",

        # images gone
        "BaseballCapsAndTiaras",
        "BiMorphon",
        "CROSSWORLDSNEXUS",
        "DevilSpy",
        "Fathead",
        "GOODBYEREPTILIANS",
        "KevinZombie",
        "KindergardenCrisIs",
        "NoSongsForTheDead",
        "RequiemShadowbornPariah",
        "SandboxDrama",
        "STICKFODDER",
        "TezzleAndZeek",
        "TheRealmOfKaerwyn",

        # broken HTML
        "CrossingOver",

        # unique html
        "IKilledTheHero",
        "PowerOfPower",
        "Schizmatic",
        "WakeTheSleepers",
        "WeightOfEternity",

        # moved
        "OopsComicAdventure",
    )

    def handle_url(self, url):
        """Parse one search result page."""
        data = self.get_url(url)

        for comicdiv in self.xpath(data, '//div[d:class("webcomic-result")]'):
            comiclink = self.xpath(comicdiv, './div[d:class("webcomic-result-title")]/a')[0]
            comicurl = comiclink.attrib['href']
            name = comiclink.text

            info = self.xpath(comicdiv, './/span[d:class("stat-value")]')
            # find out how many images this comic has
            count = int(info[0].text.strip())
            self.add_comic(name, comicurl, count)

        nextlink = self.xpath(data, '//div[d:class("search-next-page")]/a')
        if nextlink:
            return nextlink[0].attrib['href']
        else:
            return None

    def collect_results(self):
        """Parse all search result pages."""
        # Sort by page count, so we can abort when we get under some threshold.
        url = ('https://comicfury.com/search.php?query=&lastupdate=0&' +
          'completed=1&fn=2&fv=2&fs=2&fl=2&sort=0')

        print("Parsing search result pages...", file=sys.stderr)
        while url:
            url = self.handle_url(url)

    def get_entry(self, name, entry):
        url = entry
        sub = urlsplit(url).hostname.split('.', 1)[0]
        return f"cls('{name}', '{sub}'),"


if __name__ == '__main__':
    ComicFuryUpdater(__file__).run()



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from __future__ import annotations

import _thread
import logging
import os
import re
import threading
from queue import Empty, Queue
from typing import Collection
from urllib.parse import urlparse

from . import events
from .scraper import scrapers as scrapercache

logger = logging.getLogger(__name__)


class ComicQueue(Queue):
    """The comic scraper job queue."""

    def join(self, timeout=None):
        """Blocks until all items in the Queue have been gotten and processed.

        The count of unfinished tasks goes up whenever an item is added to the
        queue. The count goes down whenever a consumer thread calls task_done()
        to indicate the item was retrieved and all work on it is complete.

        When the count of unfinished tasks drops to zero, join() unblocks.
        """
        self.all_tasks_done.acquire()
        try:
            while self.unfinished_tasks:
                self.all_tasks_done.wait(timeout)
        finally:
            self.all_tasks_done.release()

    def clear(self):
        """Remove all queue entries."""
        self.mutex.acquire()
        self.queue.clear()
        self.mutex.release()


# ensure threads download only from one host at a time
host_locks: dict[str, threading.Lock] = {}


def get_hostname(url):
    """Get hostname from URL."""
    return list(urlparse(url))[1].lower()


lock = threading.Lock()


def get_host_lock(url):
    """Get lock object for given URL host."""
    hostname = get_hostname(url)
    return host_locks.setdefault(hostname, threading.Lock())


class ComicGetter(threading.Thread):
    """Get all strips of a comic in a thread."""

    def __init__(self, options, jobs) -> None:
        """Store options."""
        super(ComicGetter, self).__init__()
        self.options = options
        self.jobs = jobs
        self.origname = self.name
        self.stopped = False
        self.errors = 0

    def run(self):
        """Process from queue until it is empty."""
        try:
            while not self.stopped:
                scraperobj = self.jobs.get(False)
                self.name = scraperobj.name
                try:
                    self.getStrips(scraperobj)
                finally:
                    self.jobs.task_done()
                    self.name = self.origname
        except Empty:
            pass
        except KeyboardInterrupt:
            _thread.interrupt_main()

    def getStrips(self, scraperobj):
        """Download comic strips."""
        with lock:
            host_lock = get_host_lock(scraperobj.url)
        with host_lock:
            self._getStrips(scraperobj)

    def _getStrips(self, scraperobj):
        """Get all strips from a scraper."""
        if self.options.numstrips:
            numstrips = self.options.numstrips
        elif self.options.cont or self.options.all:
            numstrips = None
        else:
            # get current strip
            numstrips = 1
        try:
            if scraperobj.isComplete(self.options.basepath):
                logger.info("All comics are already downloaded.")
                return 0
            for strip in scraperobj.getStrips(numstrips):
                skipped = self.saveComicStrip(strip)
                if skipped and self.options.cont:
                    # stop when retrieval skipped an image for one comic strip
                    logger.info("Stop retrieval because image file already exists")
                    break
                if self.stopped:
                    break
            if (self.options.all or
                self.options.cont) and not (self.errors or
                                            self.options.dry_run or
                                            scraperobj.indexes):
                scraperobj.setComplete(self.options.basepath)
        except Exception as msg:
            logger.exception(msg)  # noqa: LOG010 # FIXME: Legacy stuff, needs refactor
            self.errors += 1

    def saveComicStrip(self, strip):
        """Save a comic strip which can consist of multiple images."""
        allskipped = True
        for image in strip.getImages():
            try:
                if self.options.dry_run:
                    filename, saved = "", False
                else:
                    filename, saved = image.save(self.options.basepath)
                if saved:
                    allskipped = False
                if self.stopped:
                    break
            except Exception as msg:
                logger.exception('Could not save image at %r to %r: %s',
                    image.referrer, image.filename, msg)
                self.errors += 1
        return allskipped

    def stop(self):
        """Mark this thread as stopped."""
        self.stopped = True


def getComics(options):
    """Retrieve comics."""
    threads = []
    jobs = ComicQueue()

    if options.handler:
        for name in set(options.handler):
            events.addHandler(name, options.basepath, options.baseurl, options.allowdownscale)
    events.getHandler().start()
    errors = 0
    try:
        for scraperobj in getScrapers(options.comic, options.basepath,
                options.adult):
            jobs.put(scraperobj)
        # start threads
        num_threads = min(options.parallel, jobs.qsize())
        for _i in range(num_threads):
            t = ComicGetter(options, jobs)
            threads.append(t)
            t.start()
        # wait for threads to finish
        jobs.join(1)
        for t in threads:
            errors += t.errors
    except ValueError as msg:
        logger.exception(msg)  # noqa: LOG010
        errors += 1
    except KeyboardInterrupt:
        logger.warning("Interrupted! Waiting for download threads to finish.")
    finally:
        for t in threads:
            t.stop()
        jobs.clear()
        events.getHandler().end()
        events.clear_handlers()
    return errors


def getScrapers(comics: Collection[str], basepath: str, adult=True, listing=False):
    """Get scraper objects for the given comics."""
    if '@' in comics:
        # only scrapers whose directory already exists
        if len(comics) > 1:
            logger.warning("using '@' as comic name ignores all other specified comics.")
        yield from get_existing_comics(basepath, adult, listing)
    else:
        # get only selected comic scrapers
        # store them in a set to eliminate duplicates
        scrapers = set()
        basere = re.compile(r'^' + re.escape(basepath) + r'[/\\]')
        for comic in comics:
            # Helpful when using shell completion to pick comics to get
            comic = comic.rstrip(os.path.sep)
            if basere.match(comic):
                # make the following command work:
                # find Comics -type d | xargs -n1 -P10 dosage -b Comics
                comic = comic[len(basepath) + 1:].lstrip(os.sep)
            if ':' in comic:
                name, index = comic.split(':', 1)
                indexes = index.split(',')
            else:
                name = comic
                indexes = None
            scraper = scrapercache.find(name)
            if shouldRunScraper(scraper, adult, listing):
                # FIXME: Find a better way to work with indexes
                scraper.indexes = indexes
                if scraper not in scrapers:
                    scrapers.add(scraper)
                    yield scraper


def get_existing_comics(basepath=None, adult=True, listing=False):
    for scraperobj in scrapercache.all(include_removed=True):
        dirname = scraperobj.get_download_dir(basepath)
        if os.path.isdir(dirname):
            if shouldRunScraper(scraperobj, adult, listing):
                yield scraperobj


def shouldRunScraper(scraperobj, adult=True, listing=False):
    if listing:
        return True
    if not adult and scraperobj.adult:
        warn_adult(scraperobj)
        return False
    reasons = scraperobj.getDisabledReasons()
    if reasons:
        warn_disabled(scraperobj, reasons)
        return False
    return True


def warn_adult(scraperobj):
    """Print warning about adult content."""
    logger.warning("skipping adult comic %s; use the --adult option to confirm your age",
        scraperobj.name)


def warn_disabled(scraperobj, reasons):
    """Print warning about disabled comic modules."""
    logger.warning("Skipping comic %s: %s", scraperobj.name, ' '.join(reasons.values()))



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2020 Tobias Gruetzmacher

from lxml import html

from dosagelib.xml import NS

import httpmocks


tree = html.document_fromstring(httpmocks.content('zp-222'))


class TestXML:
    def xpath(self, path):
        return tree.xpath(path, namespaces=NS)

    def test_class_ext(self):
        assert len(self.xpath('//li[d:class("menu-item-3773")]')) == 1
        assert len(self.xpath('//ul[d:class("menu")]')) == 1
        assert len(self.xpath('//li[d:class("menu-item-object-custom")]')) == 2
        assert len(self.xpath('//li[d:class("menu-item")]')) == 25

    def test_class_start_ext(self):
        assert len(self.xpath('//li[d:class_start("menu-item-5")]')) == 5

    def test_re_ext(self):
        assert len(self.xpath(r'//img[re:test(@src, "posters.*jpg")]')) == 1



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from re import compile, escape

from .. import util
from ..helpers import bounceStarter, indirectStarter, joinPathPartsNamer
from ..scraper import BasicScraper, ParserScraper, _BasicScraper, _ParserScraper
from ..util import tagre
from .common import (
    ComicControlScraper,
    WordPressNavi,
    WordPressScraper,
    WordPressWebcomic,
)


class Damonk(_BasicScraper):
    url = 'http://www.damonk.com/'
    stripUrl = url + 'd/%s.html'
    firstStripUrl = stripUrl % '20060522'
    imageSearch = compile(tagre("img", "src", r'(/comics/[^"]+)'))
    prevSearch = compile(tagre("a", "href", r'(/d/\d+\.html)') +
                         tagre("img", "src", r'/images/previous_day\.gif'))
    help = 'Index format: yyyymmdd'


class DangerouslyChloe(ComicControlScraper):
    url = 'http://www.dangerouslychloe.com/'
    firstStripUrl = url + 'strips-dc/Chapter_1_-_That_damned_girl'


class DarkLegacy(ParserScraper):
    starter = indirectStarter
    url = "https://www.darklegacycomics.com"
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//div[@class="comic narrow"]//img'
    prevSearch = ('//a[@title="Previous - A"]',
                  '//a[@title="Previous"]')
    latestSearch = '//div[@class="comic narrow feed"]//a'
    help = 'Index format: n (unpadded)'


class DarkWhite(WordPressScraper):
    url = 'https://www.darkwhitecomic.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'chapter-1-sleep'


class DarthsAndDroids(_BasicScraper):
    url = 'http://www.darthsanddroids.net/'
    stripUrl = url + 'episodes/%s.html'
    firstStripUrl = stripUrl % '0001'
    prevSearch = compile(tagre("a", "href", r'(/episodes/\d\d\d\d.html)') +
                         '&lt;PREVIOUS')
    imageSearch = compile(tagre("img", "src", r'(/comics/darths\d\d\d\d\.jpg)'))


class DasLebenIstKeinPonyhof(_ParserScraper):
    baseUrl = 'https://sarahburrini.com/comic/das-leben-ist-kein-ponyhof/'
    url = baseUrl + 'und-nu/'
    firstStripUrl = url + 'mein-erster-webcomic/'
    imageSearch = '//img[d:class("attachment-full")]'
    prevSearch = '//a[@rel="prev"]'
    endOfLife = True
    lang = 'de'


class DaughterOfTheLilies(ComicControlScraper):
    url = 'https://www.daughterofthelilies.com/'
    firstStripUrl = url + 'dotl/part-1-a-girl-with-no-face'


class DeadWinter(_BasicScraper):
    url = 'http://deadwinter.cc/'
    stripUrl = url + 'page/%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(tagre("img", "src", r"(/static/page/strip/\d+[^']+)", quote="'"))
    prevSearch = compile(tagre("a", "href", r'(/page/\d+)') + "Previous")
    help = 'Index format: number'


class Deathbulge(BasicScraper):
    url = 'https://www.deathbulge.com/api/comics'
    imageSearch = compile(r"(/images/comics/[^\.]+\.jpg)")
    prevSearch = compile(r'"previous":(\d+),')
    firstStripUrl = url + '/1'

    def getPrevUrl(self, url, data):
        if data[1] == self.url:
            data = (data[0], data[1] + '/')
        return super().getPrevUrl(url, data)


class DeepFried(_BasicScraper):
    url = 'http://www.whatisdeepfried.com/'
    rurl = escape(url)
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2001/09/16/new-world-out-of-order'
    imageSearch = compile(tagre("img", "src", r'(%scomics/[^"]+)' % rurl))
    prevSearch = compile(tagre("a", "href", r'(%s[^"]+)' % rurl, after="prev"))
    help = 'Index format: none'


class DeerMe(_ParserScraper):
    url = 'https://www.deerme.net/'
    firstStripUrl = url + 'a.php?b=comic/c001-image/dm-20031121-001-0001'
    imageSearch = '//div[d:class("ComicImage")]/img'
    prevSearch = '//a[@rel="prev"]'
    nextSearch = '//a[@rel="next"]'
    starter = bounceStarter


class Delve(WordPressScraper):
    url = 'https://thisis.delvecomic.com/NewWP/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'in-too-deep'
    adult = True
    maxLen = len('episode999')

    def namer(self, imageUrl, pageUrl):
        # Fix inconsistent filenames
        filename = util.urlpathsplit(imageUrl)[-1]
        if (pageUrl == self.stripUrl % 'engagement' or
                pageUrl == self.stripUrl % 'losing-it'):
            self.maxLen = self.maxLen - 1
        if ('episode' in filename and
                len(filename) - len('.jpg') > self.maxLen and
                filename[self.maxLen] != '-'):
            filename = filename[:self.maxLen] + '-' + filename[self.maxLen:]
        return filename


class DemolitionSquad(_ParserScraper):
    url = 'http://www.demolitionsquad.de/'
    stripUrl = url + '?comicbeitrag=%s'
    firstStripUrl = stripUrl % '181'
    imageSearch = '//img[contains(@src,"uploads/pics/")]'
    prevSearch = '//img[@name="zuruck"]/..'
    help = 'Index format: number'
    lang = 'de'


class DerTodUndDasMaedchen(_ParserScraper):
    url = ('https://web.archive.org/web/20180106180134/'
        'http://www.cartoontomb.de/deutsch/tod2.php')
    stripUrl = url + '?bild=%s.jpg'
    firstStripUrl = stripUrl % '00_01_01'
    imageSearch = '//img[contains(@src, "images/tod/teil2")]'
    prevSearch = u'//a[text()="zur\u00FCck"]'
    help = 'Index format: nn_nn_nn'
    lang = 'de'


class DesertFox(WordPressWebcomic):
    url = 'https://desertfoxcomics.net/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'origins-1'

    def namer(self, image_url, page_url):
        # Fix inconsistent filenames
        filename = util.urlpathsplit(image_url)[-1]
        filename = filename.replace('Pg', 'Page').replace('Desert-Fox', '')
        if 'origins' in page_url:
            filename = filename.replace('Page-', 'Page-0-')
        return filename


class DieFruehreifen(_BasicScraper):
    url = 'http://www.die-fruehreifen.de/index.php'
    stripUrl = url + '?id=%s&order=DESC'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(tagre("img", "src", r'([^"]*/strips/[Ff]rueh_?[Ss]trip_\d+.jpg)'))
    prevSearch = compile(tagre("a", "href", r"(index\.php\?id=\d+&order=DESC)") +
                         tagre("img", "id", r"naechster"))
    help = 'Index format: n (unpadded)'
    lang = 'de'


class DieselSweeties(_ParserScraper):
    url = 'http://dieselsweeties.com/'
    stripUrl = url + 'ics/%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[@class="xomic"]'
    prevSearch = '//div[@id="prev"]//a[contains(text(), "previous")]'
    latestSearch = prevSearch
    starter = indirectStarter
    help = 'Index format: n (unpadded)'


class DieselSweetiesOld(_ParserScraper):
    url = 'http://dieselsweeties.com/archive/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src, "/hstrips/")]'
    prevSearch = '//a[contains(@title, "previous")]'
    help = 'Index format: n (unpadded)'
    endOfLife = True

    def starter(self):
        return self.stripUrl % '4000'

    def namer(self, image_url, page_url):
        index = int(util.urlpathsplit(image_url)[-1].split('.')[0])
        return 'sw%02d' % index


class DocRat(WordPressWebcomic):
    url = 'https://www.docrat.com.au/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'begin-with-eye-contact'

    def namer(self, imageUrl, pageUrl):
        # Fix inconsistent filenames
        filename = util.urlpathsplit(imageUrl)[-1]
        filename = filename.replace('2006-08-01', 'DR0027')
        filename = filename.replace('2006-07-31', 'DR0026')
        return filename


class DoemainOfOurOwn(ParserScraper):
    url = 'http://www.doemain.com/'
    stripUrl = url + 'html/%s.html'
    firstStripUrl = stripUrl % '1999/1999-04-24'
    imageSearch = '//img[contains(@src, "strips/")]'
    prevSearch = '//a[img[@alt="Previous Strip"]]'
    endOfLife = True
    help = 'Index format: yyyy-mm-dd'

    def namer(self, image_url, page_url):
        # Fix date formatting
        filename = util.urlpathsplit(image_url)[-1]
        if len(filename) > 6 and filename[0:6].isdigit():
            month = filename[0:2]
            day = filename[2:4]
            year = ('19' if filename[4] == '9' else '20') + filename[4:6]
            filename = '%s-%s-%s%s' % (year, month, day, filename[6:])
        return filename


class DoesNotPlayWellWithOthers(WordPressNavi):
    url = 'http://www.doesnotplaywellwithothers.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'pwc-0001'
    adult = True


class DoghouseDiaries(ParserScraper):
    url = 'http://thedoghousediaries.com/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '34'
    imageSearch = '//img[@class="imgcomic"]'
    textSearch = imageSearch + '/@title'
    prevSearch = '//a[@id="previouslink"]'
    nextSearch = '//a[@id="nextlink"]'
    starter = bounceStarter
    namer = joinPathPartsNamer(pageparts=(-1,))
    help = 'Index format: number'


class DominicDeegan(_ParserScraper):
    url = 'https://www.dominic-deegan.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % '0001-20020521'
    imageSearch = '//img[contains(@class, "wp-post-image")]'
    prevSearch = '//a[@title="Prev"]'
    help = 'Index format: ####-yyyymmdd'


class DorkTower(_ParserScraper):
    url = 'http://www.dorktower.com/'
    firstStripUrl = url + '1997/01/01/shadis-magazine-strip-1/'
    imageSearch = '//div[d:class("entry-content")]//a/img'
    prevSearch = '//a[d:class("btn")][text()="Previous"]'


class DoomsdayMyDear(_ParserScraper):
    url = 'http://doomsdaymydear.com/'
    imageSearch = '//img[d:class("attachment-full")]'
    prevSearch = '//a[d:class("previous-webcomic-link")]'


class Draconia(WordPressWebcomic):
    url = 'https://www.draconiachronicles.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'chapter-1-page-1'


class Dracula(_BasicScraper):
    url = 'http://draculacomic.net/'
    stripUrl = url + 'comic.php?comicID=%s'
    firstStripUrl = stripUrl % '0'
    imageSearch = compile(tagre("img", "src", r'(comics/[^"]+)'))
    prevSearch = compile(r'&nbsp;<a class="archivelink" href="(.+?)">&laquo; Prev</a>')
    help = 'Index format: nnn'


class DreamKeepers(_ParserScraper):
    url = 'http://www.dreamkeeperscomic.com/GNSaga.php'
    stripUrl = url + '?pg=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src, "GNSagapages")]'
    prevSearch = '//a[@id="prev"]'
    help = 'Index format: n'


class DreamKeepersPrelude(_ParserScraper):
    url = 'http://www.dreamkeeperscomic.com/Prelude.php'
    stripUrl = url + '?pg=%s'
    firstStripUrl = stripUrl % '0001'
    imageSearch = '//div[@class="Preludecomic"]/table//a/img'
    prevSearch = '//a[@id="prev"]'
    help = 'Index format: n'


class DresdenCodak(ParserScraper):
    url = 'http://dresdencodak.com/'
    firstStripUrl = url + '2007/02/08/pom/'
    imageSearch = '//section[d:class("entry-content")]//img[d:class("aligncenter")]'
    prevSearch = '//a[img[contains(@src, "prev")]]'
    latestSearch = '//a[d:class("tc-grid-bg-link")]'
    starter = indirectStarter


class DrFun(_ParserScraper):
    baseUrl = ('https://web.archive.org/web/20180726145737/'
        'http://www.ibiblio.org/Dave/')
    stripUrl = baseUrl + 'ar%s.htm'
    url = stripUrl % '00502'
    firstStripUrl = stripUrl % '00001'
    imageSearch = '//a[contains(@href, "Dr-Fun/df")]'
    multipleImagesPerStrip = True
    prevSearch = '//a[contains(text(), "Previous Week")]'
    endOfLife = True
    help = 'Index format: nnnnn'


class Drive(ParserScraper):
    url = 'http://www.drivecomic.com/'
    firstStripUrl = url + 'comic/act-1-pg-001/'
    imageSearch = ('//div[@id="unspliced-comic"]//img/@data-src-img',
        '//div[@id="unspliced-comic"]//picture//img')
    prevSearch = '//a[d:class("previous-comic")]'


class DrMcNinja(ParserScraper):
    url = ('https://web.archive.org/web/20210322033246/'
        'http://drmcninja.com/')
    stripUrl = url + 'archives/comic/%s/'
    firstStripUrl = stripUrl % '0p1'
    imageSearch = '//div[@id="comic"]/img'
    prevSearch = '//a[d:class("prev")]'
    help = 'Index format: {episode}p{page}'


class Drowtales(_ParserScraper):
    url = 'http://www.drowtales.com/mainarchive.php'
    stripUrl = url + '?sid=%s'
    firstStripUrl = stripUrl % '4192'
    imageSearch = '//div[@id="content_middle"]//img'
    prevSearch = '//a[@id="link_prev_top"]'
    help = 'Index format: number'


class DungeonsAndDenizens(WordPressNavi):
    url = ('https://web.archive.org/web/20160308001834/'
        'http://dungeond.com/')
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2005/08/23/08232005'
    endOfLife = True


class DumbingOfAge(WordPressNavi):
    url = 'http://www.dumbingofage.com/'
    stripUrl = url + '%s/'
    help = 'Index format: yyyy/comic/book-num/seriesname/stripname'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2025 Tobias Gruetzmacher
import pytest
import requests
import responses

from dosagelib import http


@responses.activate
def test_no_robotstxt():
    responses.get('http://a/robots.txt', status=404)
    http.check_robotstxt("http://a/somefile.html", requests.Session())


@responses.activate
def test_err_robotstxt():
    responses.get('http://b/robots.txt', body=IOError("Timeout!"))
    http.check_robotstxt("http://b/somefile.html", requests.Session())


@responses.activate
def test_empty_robotstxt():
    responses.get('http://c/robots.txt')
    http.check_robotstxt("http://c/somefile.html", requests.Session())


@responses.activate
def test_allowed_robotstxt():
    responses.get('http://d/robots.txt', body="User-agent: *\nDisallow:")
    http.check_robotstxt("http://d/somefile.html", requests.Session())


@responses.activate
def test_all_denied_robotstxt():
    responses.get('http://e/robots.txt', body="User-agent: *\nDisallow: /")
    http.check_robotstxt("http://e/somefile.html", requests.Session())


@responses.activate
def test_explit_denied_robotstxt():
    responses.get('http://f/robots.txt', body="User-agent: dosage\nDisallow: /")
    with pytest.raises(IOError):  # noqa: PT011 # FIXME: Refactor code to use another exception?
        http.check_robotstxt("http://f/somefile.html", requests.Session())



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
# Copyright (C) 2019-2020 Daniel Ring
from .common import WordPressScraper, WordPressWebcomic


class YAFGC(WordPressScraper):
    baseUrl = 'https://www.yafgc.net/'
    url = baseUrl + '?latest'
    stripUrl = baseUrl + 'comic/%s'
    firstStripUrl = stripUrl % 'bob-meets-gren'

    def __init__(self, name):
        super().__init__(name)
        self.session.add_throttle('www.yafgc.net', 3.0, 15.5)


class YoshSaga(WordPressWebcomic):
    url = 'https://www.yoshsaga.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'introduction'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
import itertools

from ..scraper import ParserScraper
from ..helpers import indirectStarter, joinPathPartsNamer


class Derideal(ParserScraper):
    baseUrl = 'https://derideal.com/'
    imageSearch = '//img[d:class("comic-page") or d:class("comic-pag")]'
    prevSearch = '//a[text()="<"]'
    starter = indirectStarter
    namer = joinPathPartsNamer(imageparts=range(-3, 0))

    def __init__(self, name, lang, sub, first, eol=False, multi=False):
        if lang == 'en':
            base = 'Derideal'
            lateststr = 'Read latest update'
        else:
            base = 'DeridealSpanish'
            sub = f'{lang}/{sub}'
            lateststr = 'Leer última actualización'

        if not name:
            super().__init__(base)
        else:
            super().__init__(f'{base}/{name}')

        self.url = f'{self.baseUrl}{sub}'
        self.firstStripUrl = f'{self.url}/{first}/'
        self.latestSearch = f'//a[contains(text(), "{lateststr}")]'
        self.lang = lang
        self.endOfLife = eol
        self.multipleImagesPerStrip = multi

    @classmethod
    def getmodules(cls):
        return itertools.chain.from_iterable((
            cls('', lang, 'derideal', 'chimeras-cover'),
            cls('Legacy', lang, 'derideal-legacy', 'the-dream-cover', eol=True),
            cls('LostMemories', lang, 'lost-memories', 'lost-memories-pixi', multi=True),
            cls('Nova', lang, 'nova', 'xen-prelude-cover'),
            cls('ProjectPrime', lang, 'project-prime', 'custus-part-i-cover'),
            cls('PurpurinaEffect', lang, 'purpurina-effect', 'purpurina-effect-cover'),
            cls('RLE', lang, 'RLE', 'the-leyend-of-the-rose-cover'),
            cls('TheVoid', lang, 'the-void', 'the-void-cover'),
        ) for lang in ('en', 'es'))



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2016 Tobias Gruetzmacher
from PyInstaller.utils import hooks

hiddenimports = ['dosagelib.data'] + hooks.collect_submodules('dosagelib.plugins')
datas = hooks.copy_metadata('dosage') + hooks.collect_data_files('dosagelib')



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2020 Tobias Gruetzmacher
import pytest
import re
from dosagelib.util import normaliseURL, tagre, get_system_uid


class TestURL(object):
    """
    Tests for URL utility functions.
    """

    def test_normalisation(self):
        # Test URL normalisation.
        assert (normaliseURL('http://example.com//bar/baz&amp;baz') ==
            u'http://example.com/bar/baz&baz')


class TestRegex(object):

    ValuePrefix = '/bla/'

    @pytest.mark.parametrize(('tag', 'value', 'domatch'), [
        ('<img src="%s">', ValuePrefix + 'foo', True),
        ('< img  src = "%s" >', ValuePrefix, True),
        ('<img class="prev" src="%s">', ValuePrefix + '...', True),
        ('<img origsrc="%s">', ValuePrefix, False),
        ('<Img src="%s">', ValuePrefix, True),
        ('<img SrC="%s">', ValuePrefix, True),
        ('<img src="%s">', ValuePrefix[:-1], False),
        ('<img class="prev" src="%s" a="b">', ValuePrefix, True),
    ])
    def test_regex(self, tag, value, domatch):
        matcher = re.compile(tagre("img", "src", '(%s[^"]*)' %
                             self.ValuePrefix))
        self.match_tag(matcher, tag, value, domatch)

    def match_tag(self, matcher, tag, value, domatch=True):
        text = tag % value
        match = matcher.search(text)
        if domatch:
            assert match, "%s should match %s" % (matcher.pattern, text)
            assert match.group(1) == value
        else:
            assert not match, "%s should not match %s" % (matcher.pattern,
                                                          text)


class TestUid(object):
    """
    Tests for unique system IDs.
    """

    def test_system_uid(self):
        assert get_system_uid()



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2020 Tobias Gruetzmacher
from .common import WordPressScraper


class SandraAndWoo(WordPressScraper):
    prevSearch = '//a[@rel="prev"]'

    def __init__(self, name, urlName, firstUrl, lang='en'):
        super().__init__(name)
        self.url = 'http://www.sandraandwoo.com/' + urlName
        self.firstStripUrl = self.url + firstUrl
        self.lang = lang

    @classmethod
    def getmodules(cls):
        return (
            cls('Gaia', 'gaia/', '2000/01/01/welcome-to-gaia/'),
            cls('GaiaGerman', 'gaiade/', '2000/01/01/welcome-to-gaia/', lang='de'),
            cls('SandraAndWoo', '', '2000/01/01/welcome-to-sandra-and-woo/'),
            cls('SandraAndWooGerman', 'woode/',
                '2008/10/19/ein-ausgefuchster-waschbar/', lang='de'),
        )



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from __future__ import annotations

from typing import Protocol

from . import util
from .scraper import Scraper


class Namer(Protocol):
    """A protocol for generic callbacks to name web comic images."""
    def __call__(_, self: Scraper, image_url: str, page_url: str) -> str:
        ...


def queryNamer(param, use_page_url=False) -> Namer:
    """Get name from URL query part."""
    def _namer(self, image_url: str, page_url: str) -> str:
        """Get URL query part."""
        url = page_url if use_page_url else image_url
        return util.getQueryParams(url)[param][0]
    return _namer


def regexNamer(regex, use_page_url=False) -> Namer:
    """Get name from regular expression."""
    def _namer(self, image_url: str, page_url: str) -> str:
        """Get first regular expression group."""
        url = page_url if use_page_url else image_url
        mo = regex.search(url)
        return mo.group(1) if mo else util.urlpathsplit(image_url)[-1]
    return _namer


def joinPathPartsNamer(pageparts=(), imageparts=(), joinchar='_') -> Namer:
    """Get name by mashing path parts together with underscores."""
    def _namer(self: Scraper, image_url: str, page_url: str) -> str:
        # Split and drop host name
        pagesplit = util.urlpathsplit(page_url)
        imagesplit = util.urlpathsplit(image_url)
        joinparts = ([pagesplit[i] for i in pageparts] +
            [imagesplit[i] for i in imageparts])
        return joinchar.join(joinparts)
    return _namer


def bounceStarter(self):
    """Get start URL by "bouncing" back and forth one time.

    This needs the url and nextSearch properties be defined on the class.
    """
    data = self.getPage(self.url)
    prevurl = self.fetchUrl(self.url, data, self.prevSearch)
    prevurl = self.link_modifier(self.url, prevurl)
    data = self.getPage(prevurl)
    nexturl = self.fetchUrl(prevurl, data, self.nextSearch)
    return self.link_modifier(prevurl, nexturl)


def indirectStarter(self):
    """Get start URL by indirection.

    This is useful for comics where the latest comic can't be reached at a
    stable URL. If the class has an attribute 'startUrl', this page is fetched
    first, otherwise the page at 'url' is fetched. After that, the attribute
    'latestSearch' is used on the page content to find the latest strip."""
    url = self.startUrl if hasattr(self, "startUrl") else self.url
    data = self.getPage(url)
    newurl = self.fetchUrl(url, data, self.latestSearch)
    return self.link_modifier(url, newurl)



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
import logging

from .. import util
from ..scraper import ParserScraper

logger = logging.getLogger(__name__)


class Tapas(ParserScraper):
    baseUrl = 'https://tapas.io/'
    imageSearch = '//article[contains(@class, "js-episode-article")]//img/@data-src'
    prevSearch = '//a[contains(@class, "js-prev-ep-btn")]'
    latestSearch = '//ul[contains(@class, "js-episode-list")]//a'
    multipleImagesPerStrip = True

    def __init__(self, name, url):
        super().__init__('Tapas/' + name)
        self.url = self.baseUrl + 'series/' + url + '/info'
        self.stripUrl = self.baseUrl + 'episode/%s'

    def starter(self):
        # Retrieve comic metadata from info page
        info = self.getPage(self.url)
        series = self.match(info, '//@data-series-id')[0]
        # Retrieve comic metadata from API
        data = self.session.get(self.baseUrl + 'series/' + series + '/episodes?sort=NEWEST')
        data.raise_for_status()
        episodes = data.json()['data']['body']
        return self.stripUrl % episodes.split('data-id="')[1].split('"')[0]

    def getPrevUrl(self, url, data):
        # Retrieve comic metadata from API
        data = self.session.get(url + '/info')
        data.raise_for_status()
        apiData = data.json()['data']
        if apiData['scene'] == 2:
            self.firstStripUrl = self.stripUrl % apiData['prev_ep_id']
        return self.stripUrl % apiData['prev_ep_id']

    def extract_image_urls(self, url, data):
        # Save link order for position-based filenames
        self._cached_image_urls = super().extract_image_urls(url, data)
        return self._cached_image_urls

    def shouldSkipUrl(self, url, data):
        if self.match(data, '//button[d:class("js-have-to-sign")]'):
            logger.warning('Nothing to download on %r, because a login is required.', url)
            return True
        return False

    def namer(self, imageUrl, pageUrl):
        # Construct filename from episode number and image position on page
        episodeNum = util.urlpathsplit(pageUrl)[-1]
        imageNum = self._cached_image_urls.index(imageUrl)
        if len(self._cached_image_urls) > 1:
            filename = "%s-%d" % (episodeNum, imageNum)
        else:
            filename = "%s" % (episodeNum)
        return filename

    @classmethod
    def getmodules(cls):
        return (
            # Manually-added comics
            cls('AmpleTime', 'Ample-Time'),
            cls('FANGS', 'fangscomic'),
            cls('FishNuggets', 'Fish-Nuggets'),
            cls('Ginpu', 'Ginpu-Studios-Comics'),
            cls('InsignificantOtters', 'IOtters'),
            cls('MagicalBoy', 'magicalboy'),
            cls('NoFuture', 'NoFuture'),
            cls('OrensForge', 'OrensForge'),
            cls('RadioactivePanda', 'Radioactive-Panda'),
            cls('RavenWolf', 'RavenWolf'),
            cls('SyntheticInstinct', 'Synthetic-Instinct'),
            cls('TheCatTheVineAndTheVictory', 'The-Cat-The-Vine-and-The-Victory'),
            cls('TheInkApprentice', 'The-Ink-Apprentice'),
            cls('TheSeaInYou', 'theseainyou'),
            cls('TheSelkiesSkin', 'theselkiesskincomic'),
            cls('TheWitchsThrone', 'thewitchsthrone'),
            cls('VenturaCityDrifters', 'Ventura-City-Drifters'),

            # START AUTOUPDATE
            # END AUTOUPDATE
        )



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2020 Tobias Gruetzmacher
# Copyright (C) 2019-2020 Daniel Ring
from re import compile, IGNORECASE

from ..scraper import BasicScraper, ParserScraper
from .common import ComicControlScraper, WordPressScraper, WordPressNavi, WordPressWebcomic


class Kaspall(ParserScraper):
    stripUrl = 'http://www.kaspall.com/comic/%s'
    url = stripUrl % '2015/10/11'
    firstStripUrl = '2004/08/05'
    imageSearch = '//img[contains(@src, "comics/")]'
    prevSearch = '//a[./img[contains(@src, "prev_comic")]]'
    endOfLife = True


class KevinAndKell(BasicScraper):
    url = 'http://www.kevinandkell.com/'
    stripUrl = url + '%s/kk%s%s.html'
    firstStripUrl = stripUrl % ('1995', '09', '03')
    imageSearch = compile(r'<img.+?src="(/?(\d+/)?strips/kk\d+.(gif|jpg))"',
                          IGNORECASE)
    prevSearch = compile(
        r'<a.+?href="(/?(\.\./)?\d+/kk\d+\.html)"[^>]*><span>Previous Strip',
        IGNORECASE)
    help = 'Index format: yyyy-mm-dd'

    def getIndexStripUrl(self, index):
        return self.stripUrl % tuple(map(int, index.split('-')))


class KickInTheHead(WordPressNavi):
    url = 'http://www.kickinthehead.org/'
    firstStripUrl = url + '2003/03/20/ipod-envy/'


class KillSixBillionDemons(WordPressNavi):
    url = 'http://killsixbilliondemons.com/'
    firstStripUrl = url + 'comic/kill-six-billion-demons-chapter-1/'
    multipleImagesPerStrip = True
    adult = True


class Kitfox(WordPressScraper):
    url = 'http://www.kitfox.com/wordpress/'
    stripUrl = url + '?comic=%s'
    firstStripUrl = stripUrl % 'the-adventure-begins-almost'


class KiwiBlitz(ComicControlScraper):
    url = 'http://www.kiwiblitz.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'welcome-to-kb'


class Krakow(BasicScraper):
    url = 'http://www.krakow.krakowstudios.com/'
    stripUrl = url + 'archive.php?date=%s'
    firstStripUrl = stripUrl % '20081111'
    imageSearch = compile(r'<img src="(comics/.+?)"')
    prevSearch = compile(
        r'<a href="(archive\.php\?date=.+?)"><img border=0 name=previous_day')
    help = 'Index format: yyyymmdd'


class KuroShouri(WordPressWebcomic):
    url = 'http://kuroshouri.com/'
    stripUrl = url + 'kuroshouri/%s/'
    firstStripUrl = stripUrl % 'kuro-shouri'



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
from ..scraper import ParserScraper


class Arcamax(ParserScraper):
    imageSearch = '//img[@id="comic-zoom"]'
    prevSearch = '//a[@class="prev"]'

    def __init__(self, name, path) -> None:
        super().__init__('Arcamax/' + name)
        self.url = 'http://www.arcamax.com/thefunnies/' + path + '/'

    @classmethod
    def getmodules(cls):
        return (
            # do not edit anything below since these entries are generated from
            # scripts/arcamax.py
            # START AUTOUPDATE
            # 9ChickweedLane has a duplicate in GoComics/9ChickweedLane
            # Agnes has a duplicate in GoComics/Agnes
            # AndyCapp has a duplicate in GoComics/AndyCapp
            # Archie has a duplicate in Creators/Archie
            cls('ArcticCircle', 'arcticcircle'),
            # AskShagg has a duplicate in GoComics/AskShagg
            cls('BabyBlues', 'babyblues'),
            # BallardStreet has a duplicate in GoComics/BallardStreet
            # BarneyAndClyde has a duplicate in GoComics/BarneyAndClyde
            cls('BarneyGoogleAndSnuffySmith', 'barneygoogle'),
            # BC has a duplicate in GoComics/BC
            cls('BeetleBailey', 'beetlebailey'),
            cls('Bizarro', 'bizarro'),
            # BleekerTheRechargeableDog has a duplicate in GoComics/BleekerTheRechargeableDog
            cls('Blondie', 'blondie'),
            cls('Boondocks', 'boondocks'),
            cls('BrilliantMindOfEdisonLee', 'brilliantmindofedisonlee'),
            # Candorville has a duplicate in GoComics/Candorville
            cls('CarpeDiem', 'carpediem'),
            # Cathy has a duplicate in GoComics/Cathy
            # ChipBok has a duplicate in GoComics/ChipBok
            # ChuckleBros has a duplicate in GoComics/ChuckleBros
            # ClayBennett has a duplicate in GoComics/ClayBennett
            cls('Crankshaft', 'crankshaft'),
            # CulDeSac has a duplicate in GoComics/CulDeSac
            cls('Curtis', 'curtis'),
            # DaddysHome has a duplicate in GoComics/DaddysHome
            # DarrinBell has a duplicate in GoComics/DarrinBell
            cls('DennisTheMenace', 'dennisthemenace'),
            # DiamondLil has a duplicate in GoComics/DiamondLil
            cls('DinetteSet', 'thedinetteset'),
            # DogEatDoug has a duplicate in GoComics/DogEatDoug
            # DogsOfCKennel has a duplicate in GoComics/DogsOfCKennel
            # Doonesbury has a duplicate in GoComics/Doonesbury
            cls('Dustin', 'dustin'),
            cls('FamilyCircus', 'familycircus'),
            # FloAndFriends has a duplicate in GoComics/FloAndFriends
            # ForBetterOrForWorse has a duplicate in GoComics/ForBetterOrForWorse
            # ForHeavensSake has a duplicate in GoComics/ForHeavensSake
            # FortKnox has a duplicate in GoComics/FortKnox
            # FreeRange has a duplicate in GoComics/FreeRange
            # Garfield has a duplicate in GoComics/Garfield
            # GetFuzzy has a duplicate in GoComics/GetFuzzy
            # Heathcliff has a duplicate in GoComics/Heathcliff
            # HerbAndJamaal has a duplicate in GoComics/HerbAndJamaal
            cls('HiAndLois', 'hiandlois'),
            cls('JerryKingCartoons', 'humorcartoon'),
            # LisaBenson has a duplicate in GoComics/LisaBenson
            # LittleDogLost has a duplicate in GoComics/LittleDogLost
            # LongStoryShort has a duplicate in Creators/LongStoryShort
            # LooseParts has a duplicate in GoComics/LooseParts
            # Luann has a duplicate in GoComics/Luann
            cls('MallardFillmore', 'mallardfillmore'),
            cls('Marvin', 'marvin'),
            cls('MasterStrokesGolfTips', 'masterstrokes'),
            cls('MeaningOfLila', 'meaningoflila'),
            # MichaelRamirez has a duplicate in GoComics/MichaelRamirez
            # MikeDuJour has a duplicate in GoComics/MikeDuJour
            # MikeLester has a duplicate in GoComics/MikeLester
            # MikeLuckovich has a duplicate in GoComics/MikeLuckovich
            # Momma has a duplicate in GoComics/Momma
            cls('MotherGooseAndGrimm', 'mothergooseandgrimm'),
            cls('Mutts', 'mutts'),
            # NestHeads has a duplicate in GoComics/NestHeads
            # NickAnderson has a duplicate in GoComics/NickAnderson
            # NonSequitur has a duplicate in GoComics/NonSequitur
            # OneBigHappy has a duplicate in GoComics/OneBigHappy
            # Peanuts has a duplicate in GoComics/Peanuts
            # PearlsBeforeSwine has a duplicate in GoComics/PearlsBeforeSwine
            # Pickles has a duplicate in GoComics/Pickles
            # RedAndRover has a duplicate in GoComics/RedAndRover
            # ReplyAll has a duplicate in GoComics/ReplyAll
            cls('RhymesWithOrange', 'rhymeswithorange'),
            # Rubes has a duplicate in GoComics/Rubes
            # RudyPark has a duplicate in GoComics/RudyPark
            # Rugrats has a duplicate in Creators/Rugrats
            # ScaryGary has a duplicate in GoComics/ScaryGary
            # Shoe has a duplicate in GoComics/Shoe
            # SigneWilkinson has a duplicate in GoComics/SigneWilkinson
            # SpeedBump has a duplicate in GoComics/SpeedBump
            # SteveBenson has a duplicate in GoComics/SteveBenson
            # SteveBreen has a duplicate in GoComics/SteveBreen
            # StrangeBrew has a duplicate in GoComics/StrangeBrew
            cls('TakeItFromTheTinkersons', 'takeitfromthetinkersons'),
            # TheBarn has a duplicate in GoComics/TheBarn
            cls('TheLockhorns', 'thelockhorns'),
            # TheOtherCoast has a duplicate in GoComics/TheOtherCoast
            # WeePals has a duplicate in GoComics/WeePals
            # WizardOfId has a duplicate in GoComics/WizardOfId
            # WorkingItOut has a duplicate in GoComics/WorkingItOut
            # Wumo has a duplicate in GoComics/WuMo
            # ZackHill has a duplicate in GoComics/ZackHill
            cls('Zits', 'zits'),
            # END AUTOUPDATE
        )



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
import re
from operator import attrgetter

import pytest

from dosagelib.scraper import scrapers
from dosagelib.plugins import old


class TestComicNames:

    def test_names(self):
        for scraperobj in scrapers.all():
            name = scraperobj.name
            assert name.count('/') <= 1
            if '/' in name:
                comicname = name.split('/')[1]
            else:
                comicname = name
            assert re.sub("[^0-9a-zA-Z_]", "", comicname) == comicname

    @pytest.mark.parametrize(('scraperobj'),
        [obj for obj in scrapers.all(include_removed=True)
            if isinstance(obj, old.Renamed)], ids=attrgetter('name'))
    def test_renamed(self, scraperobj):
        assert len(scraperobj.getDisabledReasons()) > 0
        # Renamed scraper should only point to an non-disabled scraper
        newscraper = scrapers.find(scraperobj.newname)
        assert len(newscraper.getDisabledReasons()) == 0



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from re import compile, escape

from .. import util
from ..helpers import bounceStarter, indirectStarter, joinPathPartsNamer
from ..scraper import BasicScraper, ParserScraper
from ..util import tagre
from .common import ComicControlScraper, WordPressNaviIn, WordPressScraper


class Hackles(ParserScraper):
    url = ('https://web.archive.org/web/20220128022158/'
        'http://hackles.org/')
    stripUrl = url + 'cgi-bin/archives.pl?request=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src, "strips/")]'
    prevSearch = '//a[text()="< previous"]'
    endOfLife = True


class HagarTheHorrible(BasicScraper):
    url = 'http://www.hagarthehorrible.net/'
    stripUrl = 'http://www.hagardunor.net/comicstrips_us.php?serietype=9&colortype=1&serieno=%s'
    firstStripUrl = stripUrl % '1'
    multipleImagesPerStrip = True
    imageSearch = compile(tagre("img", "src", r'(stripus\d+/(?:Hagar_The_Horrible_?|h)\d+[^ >]+)', quote=""))
    prevUrl = r'(comicstrips_us\.php\?serietype\=9\&colortype\=1\&serieno\=\d+)'
    prevSearch = compile(tagre("a", "href", prevUrl, after="Previous"))
    help = 'Index format: number'

    def starter(self):
        """Return last gallery link."""
        url = 'http://www.hagardunor.net/comics.php'
        data = self.getPage(url)
        pattern = compile(tagre("a", "href", self.prevUrl))
        return self.fetchUrls(url, data, pattern)[-1]


class HarkAVagrant(BasicScraper):
    url = 'http://www.harkavagrant.com/'
    rurl = escape(url)
    starter = bounceStarter
    stripUrl = url + 'index.php?id=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(tagre("img", "src", r'(%s[^"]+)' % rurl,
                                after='BORDER'))
    prevSearch = compile(tagre("a", "href", r'(%sindex\.php\?id=\d+)' % rurl) +
                         tagre("img", "src", "buttonprevious.png"))
    nextSearch = compile(tagre("a", "href", r'(%sindex\.php\?id=\d+)' % rurl) +
                         tagre("img", "src", "buttonnext.png"))
    help = 'Index format: number'

    def namer(self, image_url, page_url):
        filename = util.urlpathsplit(image_url)[-1]
        num = page_url.rsplit('=', 1)[1]
        return f'{num}-{filename}'


class HavocInc(WordPressScraper):
    url = 'http://www.radiocomix.com/havoc-inc/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'havoc-cover'


class HeadlessBliss(ComicControlScraper):
    url = 'http://headlessbliss.com/'


class Hellkats(ParserScraper):
    url = 'https://poecatcomix.com/hellkatscomic/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % 'hellkats-issue-1-cover'
    imageSearch = '//img[@class="scale-with-grid wp-post-image"]'
    prevSearch = '//a[d:class("fixed-nav-prev")]'
    latestSearch = '//div[@class="post-title"]//a'
    starter = indirectStarter
    adult = True
    namer = joinPathPartsNamer(pageparts=(-2,))


class HeyFox(WordPressScraper):
    url = 'http://www.steamclaw.com/heyfox/'
    stripUrl = url + 'archives/comic/%s'
    firstStripUrl = stripUrl % '11092004'
    adult = True


class HeyKitty(WordPressScraper):
    url = 'http://heykittycomic.com/'
    stripUrl = url + '?comic=%s'
    firstStripUrl = stripUrl % 'it-begins'


class Hipsters(WordPressScraper):
    url = 'http://www.hipsters-comic.com/'
    firstStripUrl = 'http://www.hipsters-comic.com/comic/hip01/'


class HijinksEnsue(WordPressNaviIn):
    url = 'http://hijinksensue.com/'
    latestSearch = '//a[text()="Latest HijiNKS ENSUE"]'
    firstStripUrl = 'http://hijinksensue.com/comic/who-is-your-daddy-and-what-does-he-do/'
    starter = indirectStarter


class HijinksEnsueClassic(WordPressNaviIn):
    url = 'http://hijinksensue.com/comic/open-your-eyes/'
    firstStripUrl = 'http://hijinksensue.com/comic/a-soul-as-black-as-eyeliner/'
    endOfLife = True


class HijinksEnsueConvention(WordPressNaviIn):
    url = 'http://hijinksensue.com/comic/emerald-city-comicon-2015-fancy-sketches-part-4/'
    firstStripUrl = 'http://hijinksensue.com/comic/whatever-dad-im-outta-here/'
    endOfLife = True


class HijinksEnsuePhoto(WordPressNaviIn):
    url = 'http://hijinksensue.com/comic/emerald-city-comicon-2015-fancy-photo-comic-part-2/'
    firstStripUrl = 'http://hijinksensue.com/comic/san-diego-comic-con-fancy-picto-comic-pt-1/'
    endOfLife = True


class HowToBeAWerewolf(ComicControlScraper):
    url = 'https://www.howtobeawerewolf.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'coming-february-3rd'

    def namer(self, image_url, page_url):
        filename = util.urlpathsplit(image_url)[-1]
        if filename[0].isdigit():
            filename = filename.split('-', 1)[1]
        return filename



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from .. import util
from ..helpers import bounceStarter, indirectStarter
from ..scraper import ParserScraper, _ParserScraper
from .common import ComicControlScraper, WordPressScraper


class Lancer(WordPressScraper):
    url = 'https://lancercomic.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'chapter-1-cover'


class LastResort(WordPressScraper):
    url = 'http://www.lastres0rt.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'that-sound-you-hear-is-a-shattered-stereotype'


class LazJonesAndTheMayfieldRegulators(_ParserScraper):
    baseUrl = 'https://www.lazjones.com/'
    url = baseUrl + 'regulators'
    stripUrl = baseUrl + 'comic/%s'
    firstStripUrl = stripUrl % 'chapter1_00'
    imageSearch = '//img[contains(@src, "comic/pages/")]'
    prevSearch = '//a[contains(text(), "Previous")]'


class LazJonesAndTheMayfieldRegulatorsSideStories(LazJonesAndTheMayfieldRegulators):
    name = 'LazJonesAndTheMayfieldRegulators/SideStories'
    baseUrl = 'https://www.lazjones.com/'
    url = baseUrl + 'comics'
    stripUrl = baseUrl + 'comic/%s'
    firstStripUrl = stripUrl % 'journal01'

    def getPrevUrl(self, url, data):
        # Fix broken navigation links
        if url == self.url and self.match(data, self.prevSearch + '/@href')[0] == self.stripUrl % 'summer00':
            return self.stripUrl % 'summer21'
        return super(LazJonesAndTheMayfieldRegulators, self).getPrevUrl(url, data)


class LeastICouldDo(ParserScraper):
    url = 'https://leasticoulddo.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % '20030210'
    imageSearch = '//div[d:class("single-post-comic")]//img'
    prevSearch = '//a[@rel="prev"]'
    latestSearch = '//a[contains(@href, "/comic/")]'
    starter = indirectStarter
    help = 'Index format: yyyymmdd'


class LetsSpeakEnglish(ComicControlScraper):
    url = 'http://www.marycagle.com'


class LifeAintNoPonyFarm(WordPressScraper):
    url = ('https://web.archive.org/web/20181221154155/'
        'http://sarahburrini.com/en/')
    firstStripUrl = url + 'comic/my-first-webcomic/'
    multipleImagesPerStrip = True
    endOfLife = True


class LifeAsRendered(ParserScraper):
    # Reverse navigation doesn't work properly, so search forward instead
    stripUrl = 'https://kittyredden.com/LAR/%s/'
    url = stripUrl % '0100'
    firstStripUrl = stripUrl % '05extra'
    imageSearch = '//figure[@class="wp-block-image"]//img'
    prevSearch = '//a[img[@alt="Next"]]'
    textSearch = '//div[@class="entry-content"]//text()'
    adult = True
    endOfLife = True
    nav = {
        '0140': '0200',
        '0272': '02ss00',
        '02SS14': '0300',
        '0367': '03ss00',
        '03ss10': '0400',
        '0408': '0409',
        '0409': '0410',
        '0421': '0422',
        '0449': '0450',
        '0458': '0460',
        '0460': '04ss00',
        '04ss00': '04ss01',
        '04ss10': '0500',
        '0500': '0501',
        '0508': '0509',
        '0558': '0559',
        '0577': '05extra',
    }

    def namer(self, image_url, page_url):
        # Fix inconsistent filenames
        filename = util.urlpathsplit(image_url)[-1]
        return filename.replace('ReN', 'N').replace('N01P', 'A02S')

    def extract_image_urls(self, url, data):
        # Fix missing image link
        if 'LAR/0403' in url:
            return [self.stripUrl.rstrip('/') % 'A04/A04P03.png']
        return super().extract_image_urls(url, data)

    def getPrevUrl(self, url, data):
        # Fix broken navigation links
        page = util.urlpathsplit(url)[-1]
        if page in self.nav:
            return self.stripUrl % self.nav[page]
        return super().getPrevUrl(url, data)

    def fetchText(self, url, data, textSearch, optional):
        # Save final summary text
        if url == self.firstStripUrl:
            url = self.stripUrl % 'the-end'
            data = self.getPage(url)
            return super().fetchText(url, data, textSearch, optional)
        return None


class LilithsWord(ComicControlScraper):
    url = 'https://www.lilithword.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'prologue-page-00'

    def namer(self, image_url, page_url):
        return util.urlpathsplit(image_url)[-1].split('-', 1)[1]


class LittleGamers(ParserScraper):
    url = 'https://www.little-gamers.com/'
    firstStripUrl = url + '2000/12/01/99'
    imageSearch = '//div[d:class("comic")]//img'
    prevSearch = ('//a[@id="previous"]',
        '//div[d:class("comic-navigation")]//a[text()="previous"]')


class LittleTales(_ParserScraper):
    url = 'http://www.little-tales.com/'
    stripUrl = url + 'index.php?Strip=%s'
    firstStripUrl = stripUrl % '1'
    url = stripUrl % '450'
    imageSearch = '//img[contains(@src, "strips/")]'
    prevSearch = '//a[./img[@alt="BACK"]]'
    nextSearch = '//a[./img[@alt="FORWARD"]]'
    starter = bounceStarter
    nav = {
        '517': '515',
        '449': '447',
    }

    def namer(self, imageUrl, pageUrl):
        page = pageUrl.rsplit('=', 1)[-1]
        ext = imageUrl.rsplit('.', 1)[-1]
        return page + '.' + ext

    def getPrevUrl(self, url, data):
        # Skip missing pages with broken navigation links
        page = url.rsplit('=', 1)[1]
        if page in self.nav:
            return self.stripUrl % self.nav[page]
        return super(LittleTales, self).getPrevUrl(url, data)


class LoadingArtist(_ParserScraper):
    url = 'https://loadingartist.com/'
    firstStripUrl = url + 'comic/born/'
    imageSearch = '//div[d:class("main-image-container")]//img'
    prevSearch = '//div[d:class("left-nav")]//a'
    latestSearch = '//nav//a[text()="Comic"]'
    starter = indirectStarter


class LoFiJinks(WordPressScraper):
    baseUrl = 'https://hijinksensue.com/comic/'
    url = baseUrl + 'learning-to-love-again/'
    firstStripUrl = baseUrl + 'lo-fijinks-everything-i-know-anout-james-camerons-avatar-movie/'
    endOfLife = True


class LookingForGroup(ParserScraper):
    url = 'https://www.lfg.co/'
    stripUrl = url + 'page/%s/'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//div[@id="comic-img"]//img'
    prevSearch = '//a[d:class("comic-nav-prev")]'
    latestSearch = '//div[@id="feature-lfg-footer"]/a[contains(@href, "page/")]'
    starter = indirectStarter
    help = 'Index format: nnn'

    def namer(self, imageUrl, pageUrl):
        page = util.urlpathsplit(pageUrl)[-1]
        return page.replace('2967', '647')



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2016 Tobias Gruetzmacher
"""
Functions to load plugin modules.

Example usage:
    for module in loader.get_plugin_modules():
        plugins.extend(loader.get_module_plugins(module, PluginClass))
"""
import importlib
import logging
import pkgutil
import sys

from .plugins import __name__ as plugin_package
from .plugins import __path__ as plugin_path

logger = logging.getLogger(__name__)


def get_plugin_modules():
    """Find (and import) all valid modules in the "plugins" package.
    @return: all loaded valid modules
    @rtype: iterator of module
    """
    prefix = plugin_package + "."
    modules = {m[1] for m in pkgutil.iter_modules(plugin_path, prefix)}

    for elm in _get_all_modules_pyinstaller():
        if elm.startswith(prefix):
            modules.add(elm)

    for name in modules:
        try:
            yield importlib.import_module(name)
        except ImportError:
            logger.exception("could not load module %s", name)


def _get_all_modules_pyinstaller():
    # Special handling for PyInstaller
    toc = set()
    importers = pkgutil.iter_importers(__package__)
    for i in importers:
        if hasattr(i, 'toc'):
            toc |= i.toc
    return toc


def get_plugin_modules_from_dir(path, prefix='user_'):
    """Load and import a directory of python files as if they were part of the
    "plugins" package. (Mostly "stolen" from
    https://docs.python.org/3/library/importlib.html#importing-a-source-file-directly)
    """
    modules = []
    for f in path.glob('*.py'):
        name = plugin_package + "." + prefix + f.stem
        # FIXME: Drop str() when this is Python 3.6+
        spec = importlib.util.spec_from_file_location(name, str(f))
        module = importlib.util.module_from_spec(spec)
        sys.modules[name] = module
        spec.loader.exec_module(module)
        modules.append(module)
    return modules


def get_module_plugins(module, classobj):
    """Return all subclasses of a class in the module.
    If the module defines __all__, only those entries will be searched,
    otherwise all objects not starting with '_' will be searched.
    """
    try:
        names = module.__all__
    except AttributeError:
        names = [x for x in vars(module) if not x.startswith('_')]
    for name in names:
        try:
            obj = getattr(module, name)
        except AttributeError:
            continue
        try:
            if issubclass(obj, classobj):
                yield obj
        except TypeError:
            continue



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
from ..scraper import ParserScraper
from ..helpers import indirectStarter


class Creators(ParserScraper):
    imageSearch = '//a[contains(@class,"fancybox")]/img'
    prevSearch = '//a[@id="nav_prev"]'
    latestSearch = '//div[contains(@class,"caption")]/a'
    starter = indirectStarter

    def __init__(self, name, path, lang=None):
        super(Creators, self).__init__('Creators/' + name)
        self.url = 'https://www.creators.com/features/' + path
        if lang:
            self.lang = lang

    @classmethod
    def getmodules(cls):
        return (
            # Some comics are not listed on the "all" page (too old?)
            cls('CafeconLeche', 'cafe-con-leche'),
            cls('DonaldDuck', 'donald-duck'),
            cls('Flare', 'flare'),
            cls('FlightDeck', 'flight-deck'),
            cls('GirlsAndSports', 'girls-and-sports'),
            cls('HomeOffice', 'stay-at-home-dad'),
            cls('HopeAndDeath', 'hope-and-death'),
            cls('MickeyMouse', 'mickey-mouse'),
            cls('NaturalSelection', 'natural-selection'),
            cls('OffCenter', 'off-center'),
            cls('Rugrats', 'rugrats'),
            cls('TheQuigmans', 'the-quigmans'),
            cls('WinnieThePooh', 'winnie-the-pooh'),
            # do not edit anything below since these entries are generated from
            # scripts/creators.py
            # START AUTOUPDATE
            # Agnes has a duplicate in GoComics/Agnes
            # AndyCapp has a duplicate in GoComics/AndyCapp
            cls('AndyMarlette', 'andy-marlette'),
            cls('Archie', 'archie'),
            cls('ArchieSpanish', 'archie-spanish', 'es'),
            # AskShagg has a duplicate in GoComics/AskShagg
            # BallardStreet has a duplicate in GoComics/BallardStreet
            # BC has a duplicate in GoComics/BC
            # BobGorrell has a duplicate in GoComics/BobGorrell
            # ChipBok has a duplicate in GoComics/ChipBok
            # ChrisBritt has a duplicate in GoComics/ChrisBritt
            # ChuckleBros has a duplicate in GoComics/ChuckleBros
            # DaddysHome has a duplicate in GoComics/DaddysHome
            # DiamondLil has a duplicate in GoComics/DiamondLil
            # DogEatDoug has a duplicate in GoComics/DogEatDoug
            # DogsOfCKennel has a duplicate in GoComics/DogsOfCKennel
            # FloAndFriends has a duplicate in GoComics/FloAndFriends
            # ForHeavensSake has a duplicate in GoComics/ForHeavensSake
            # FreeRange has a duplicate in GoComics/FreeRange
            # GaryMarkstein has a duplicate in GoComics/GaryMarkstein
            # GaryVarvel has a duplicate in GoComics/GaryVarvel
            # Heathcliff has a duplicate in GoComics/Heathcliff
            cls('HeathcliffSpanish', 'heathcliff-spanish', 'es'),
            # HerbAndJamaal has a duplicate in GoComics/HerbAndJamaal
            # JohnDeering has a duplicate in GoComics/JohnDeering
            # KenCatalino has a duplicate in GoComics/KenCatalino
            # LibertyMeadows has a duplicate in GoComics/LibertyMeadows
            cls('LongStoryShort', 'long-story-short'),
            # MarshallRamsey has a duplicate in GoComics/MarshallRamsey
            # MichaelRamirez has a duplicate in GoComics/MichaelRamirez
            # MikeLuckovich has a duplicate in GoComics/MikeLuckovich
            # Momma has a duplicate in GoComics/Momma
            cls('Mossprints', 'mossprints'),
            # NestHeads has a duplicate in GoComics/NestHeads
            # OneBigHappy has a duplicate in GoComics/OneBigHappy
            # PaulSzep has a duplicate in GoComics/PaulSzep
            # Rubes has a duplicate in GoComics/Rubes
            # ScaryGary has a duplicate in GoComics/ScaryGary
            # SpeedBump has a duplicate in GoComics/SpeedBump
            # SteveBenson has a duplicate in GoComics/SteveBenson
            # SteveBreen has a duplicate in GoComics/SteveBreen
            # SteveKelley has a duplicate in GoComics/SteveKelley
            # StrangeBrew has a duplicate in GoComics/StrangeBrew
            # TheBarn has a duplicate in GoComics/TheBarn
            # TheMeaningOfLila has a duplicate in GoComics/TheMeaningOfLila
            # TheOtherCoast has a duplicate in GoComics/TheOtherCoast
            cls('TomStiglich', 'tom-stiglich'),
            # WeePals has a duplicate in GoComics/WeePals
            # WizardOfId has a duplicate in GoComics/WizardOfId
            cls('WizardOfIdSpanish', 'wizard-of-id-spanish', 'es'),
            # WorkingItOut has a duplicate in GoComics/WorkingItOut
            # ZackHill has a duplicate in GoComics/ZackHill
            # END AUTOUPDATE
        )



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
import json
from re import IGNORECASE, compile

from ..helpers import indirectStarter, joinPathPartsNamer
from ..scraper import ParserScraper, _BasicScraper, _ParserScraper
from ..util import tagre
from .common import ComicControlScraper, WordPressScraper, WordPressWebcomic


class MacHall(ComicControlScraper):
    url = 'https://www.machall.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'moving-in'


class MadamAndEve(_BasicScraper):
    url = 'http://www.madamandeve.co.za/'
    stripUrl = None
    imageSearch = compile(tagre('img', 'src', r'(/cartoons/me\d{6}\.(gif|jpg))'))
    multipleImagesPerStrip = True


class Magellan(WordPressScraper):
    url = 'https://magellanverse.com/'
    firstStripUrl = url + 'comic/20040307wannabe/'


class MagickChicks(ComicControlScraper):
    url = 'https://pixietrixcomix.com/magick-chicks/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % 'tis-but-a-trifle-2'
    help = 'Index format: name'
    endOfLife = True


class ManlyGuysDoingManlyThings(WordPressScraper):
    url = 'https://thepunchlineismachismo.com/'
    stripUrl = url + 'archives/comic/%s'
    firstStripUrl = stripUrl % '02222010'
    help = 'Index format: ddmmyyyy'


class MareInternum(WordPressScraper):
    url = 'https://www.marecomic.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'intro-page-1'


class Marilith(ParserScraper):
    url = 'https://web.archive.org/web/20170619193143/http://www.marilith.com/'
    stripUrl = url + 'archive.php?date=%s'
    firstStripUrl = stripUrl % '20041215'
    imageSearch = '//img[contains(@src, "comics/")]'
    prevSearch = '//a[img[@name="previous_day"]]'
    help = 'Index format: yyyymmdd'


class MarriedToTheSea(ParserScraper):
    url = 'http://marriedtothesea.com/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '022806'
    imageSearch = '//div[d:class("jumbotron")]//p/img'
    prevSearch = '//a[contains(text(), "Yesterday")]'
    namer = joinPathPartsNamer(imageparts=range(2), joinchar='-')
    help = 'Index format: mmddyy'


class MarryMe(ParserScraper):
    stripUrl = 'http://marryme.keenspot.com/d/%s.html'
    url = stripUrl % '20191001'
    firstStripUrl = stripUrl % '20120730'
    imageSearch = '//img[@class="ksc"]'
    prevSearch = '//a[@rel="prev"]'
    endOfLife = True
    help = 'Index format: yyyymmdd'


class Meek(WordPressScraper):
    url = 'https://www.meekcomic.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'chapter-1-cover'


class MegaTokyo(_BasicScraper):
    url = 'https://megatokyo.com/'
    stripUrl = url + 'strip/%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(r'"(strips/.+?)"', IGNORECASE)
    prevSearch = compile(r'"(./strip/\d+?)">Prev')
    help = 'Index format: nnnn'


class Meiosis(WordPressScraper):
    url = 'http://meiosiswebcomic.com/'


class Melonpool(WordPressScraper):
    url = 'http://www.melonpool.com/'
    allow_errors = (500,)


class MenageA3(ComicControlScraper):
    adult = True
    url = 'https://pixietrixcomix.com/menage-a-3/'
    firstStripUrl = url + 'for-new-readers'
    endOfLife = True


class Metacarpolis(ComicControlScraper):
    url = 'http://www.metacarpolis.com'


class Misfile(ComicControlScraper):
    url = 'http://www.misfile.com/misfile/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '2004-02-22'
    endOfLife = True
    help = 'Index format: yyyy-mm-dd'


class MisfileHellHigh(Misfile):
    name = 'Misfile/HellHigh'
    url = 'http://www.misfile.com/hell-high/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '2019-08-29'
    help = 'Index format: yyyy-mm-dd'


class MistyTheMouse(ParserScraper):
    url = 'http://www.mistythemouse.com/'
    imageSearch = '//center/p/img'
    prevSearch = '//a[img[contains(@src, "Previous")]]'
    firstStripUrl = url + 'The_Live_In.html'


class MonkeyUser(ParserScraper):
    url = 'https://www.monkeyuser.com/'
    imageSearch = '//div[d:class("content")]/p/img'
    prevSearch = '//a[d:class("link-reverse")]'
    latestSearch = '//div[d:class("comic")]/a'
    starter = indirectStarter

    def shouldSkipUrl(self, url, data):
        # videos
        return self.match(data, '//div[d:class("video-container")]')


class MonsieurLeChien(ParserScraper):
    url = ('https://web.archive.org/web/20210311002403/'
        'http:/www.monsieur-le-chien.fr/')
    stripUrl = url + 'index.php?planche=%s'
    firstStripUrl = stripUrl % '2'
    lang = 'fr'
    imageSearch = '//img[contains(@src,"i/planches/")]'
    prevSearch = '//a[img[contains(@src,"i/precedent.gif")]]'
    endOfLife = True
    help = 'Index format: n'


class MonsterSoup(WordPressScraper):
    url = 'https://monstersoupcomic.com/'
    stripUrl = url + '?comic=%s'
    firstStripUrl = stripUrl % 'chapter-1-cover'


class Moonlace(WordPressWebcomic):
    url = 'https://moonlace.darkbluecomics.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'page-0-1'
    adult = True

    def starter(self):
        # Set age-gate cookie
        self.session.cookies.set('age_gate', '1', domain='moonlace.darkblueworkshop.com')
        return indirectStarter(self)


class Moonsticks(ParserScraper):
    url = "https://moonsticks.org/"
    imageSearch = "//div[d:class('entry-content')]//img"
    prevSearch = ('//a[@rel="prev"]', "//a[text()='\u00AB Prev']")


class MyLifeWithFel(ParserScraper):
    baseUrl = 'https://www.mylifewithfel.com/'
    stripUrl = baseUrl + 'api/posts/%s'
    firstStripUrl = stripUrl % '1'
    url = firstStripUrl
    namer = joinPathPartsNamer(pageparts=(-1,))
    adult = True

    def starter(self):
        # Retrieve comic metadata from API
        data = self.session.get(self.url)
        data.raise_for_status()
        return self.stripUrl % data.json()['last']['id']

    def getPrevUrl(self, url, data):
        return self.stripUrl % json.loads(data.text_content())['previous']['id']

    def extract_image_urls(self, url, data):
        return [self.baseUrl + json.loads(data.text_content())['post']['image']]


class MynarskiForest(_ParserScraper):
    stripUrl = 'http://mynarskiforest.purrsia.com/xsl%s.htm'
    url = stripUrl % '09_36'
    firstStripUrl = stripUrl % '97_01'
    imageSearch = '//img[not(contains(@src, "arrow"))]'
    prevSearch = '//a[./img[contains(@src, "arrowbk")]]'
    multipleImagesPerStrip = True
    endOfLife = True


class MysteriesOfTheArcana(_ParserScraper):
    url = 'http://mysteriesofthearcana.com/'
    imageSearch = '//div[@id="comic"]//img'
    prevSearch = '//a[@class="navprevious"]'


class MonsterUnderTheBed(WordPressScraper):
    url = 'http://themonsterunderthebed.net/'
    stripUrl = url + '?comic=%s'
    firstStripUrl = stripUrl % 'test-post'
    adult = True



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
# Copyright (C) 2019-2020 Daniel Ring
from ..scraper import ParserScraper


class QuantumVibe(ParserScraper):
    url = 'https://www.quantumvibe.com/'
    stripUrl = url + 'strip?page=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src, "disppageV3?story=qv")]'
    prevSearch = '//a[./img[@alt="Previous Strip"]]'


class QuestionableContent(ParserScraper):
    url = 'http://www.questionablecontent.net/'
    stripUrl = url + 'view.php?comic=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src, "comics/")]'
    prevSearch = '//a[text()="Previous"]'
    help = 'Index format: n (unpadded)'


class Qwantz(ParserScraper):
    url = 'http://www.qwantz.com/index.php'
    stripUrl = url + '?comic=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[d:class("comic")]'
    prevSearch = '//a[@rel="prev"]'
    help = 'Index format: n'



# SPDX-License-Identifier: MIT
# Copyright (C) 2019-2022 Tobias Gruetzmacher
# Copyright (C) 2019-2020 Daniel Ring
from ..helpers import bounceStarter
from ..scraper import ParserScraper


class DMFA(ParserScraper):
    url = 'http://www.missmab.com/'
    stripUrl = url + 'Comics/%s.php'
    imageSearch = '//center//img'
    prevSearch = '//a[./img[contains(@src, "comicprev")]]'
    nextSearch = '//a[./img[contains(@src, "comicnext")]]'
    starter = bounceStarter

    def __init__(self, name, first, last=None):
        if name == 'DMFA':
            super(DMFA, self).__init__(name)
        else:
            super(DMFA, self).__init__('DMFA/' + name)

        self.firstStripUrl = self.stripUrl % first

        if last:
            self.url = self.stripUrl % last
            self.starter = super(DMFA, self).starter
            self.endOfLife = True

    @classmethod
    def getmodules(cls):
        return (
            cls('AbelsStory', 'Abel_01', last='Ab_106'),
            cls('AprilFools', 'Vol_Fools001', last='Vol_Foolslast'),
            cls('Bonus', 'Vol_Bonus001', last='Vol_Bonuslast'),
            cls('Christmas', 'HJ_01', last='HJ_06'),
            cls('DMFA', 'Vol_001'),
            cls('Guest', 'Vol_Guest001', last='Vol_Guestlast'),
            cls('Matilda', 'Ma_001', last='Ma_060'),
            cls('PerfectDate', 'PD_01', last='PD_18'),
            cls('TakePride', 'P_01', last='P_08'),
            cls('Valentines', 'Vol_VDay001', last='Vol_VDaylast'),
        )



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
from ..scraper import ParserScraper


class Nitrocosm(ParserScraper):
    imageSearch = '//img[@class="gallery_display"]'
    prevSearch = '//a[@class="nav_btn_previous"]'

    def __init__(self, name, path):
        super().__init__(name)
        self.url = 'http://www.nitrocosm.com/go/' + path

    @classmethod
    def getmodules(cls):
        return [
            cls('2214', '2214_classic/'),
            cls('OTE', 'ote/'),
            cls('ProperBarn', 'gag/'),
        ]



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring

from ..scraper import ParserScraper, _ParserScraper
from ..helpers import bounceStarter, indirectStarter


class Vexxarr(_ParserScraper):
    baseUrl = 'http://www.vexxarr.com/'
    url = baseUrl + 'Index.php'
    stripUrl = baseUrl + 'archive.php?seldate=%s'
    firstStripUrl = stripUrl % '010105'
    imageSearch = '//p/img'
    prevSearch = '//a[./img[contains(@src, "previous")]]'
    nextSearch = '//a[./img[contains(@src, "next")]]'
    starter = bounceStarter

    def namer(self, imageUrl, pageUrl):
        page = pageUrl.rsplit('=', 1)[-1]
        return '20%s-%s-%s' % (page[4:6], page[0:2], page[2:4])


class VGCats(_ParserScraper):
    url = 'https://www.vgcats.com/comics/'
    stripUrl = url + '?strip_id=%s'
    firstStripUrl = stripUrl % '0'
    imageSearch = '//td/font/img[contains(@src, "images/")]'
    prevSearch = '//a[img[contains(@src, "back.")]]'
    help = 'Index format: n (unpadded)'


class Vibe(ParserScraper):
    starter = indirectStarter
    url = 'http://www.vibecomic.com/vibe/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % ''
    imageSearch = '//div[@id="cc-comicbody"]//img'
    prevSearch = '//a[@rel="prev"]'
    nextSearch = '//a[@rel="next"]'
    latestSearch = '//a[@class="last"]'
    help = 'Index format: VIBEnnn (padded)'


class VickiFox(ParserScraper):
    url = 'http://www.vickifox.com/comic/strip'
    stripUrl = url + '?id=%s'
    firstStripUrl = stripUrl % '001'
    imageSearch = '//img[contains(@src, "comic/")]'
    prevSearch = '//button[@id="btnPrev"]/@value'

    def link_modifier(self, fromurl, tourl):
        return self.stripUrl % tourl


class ViiviJaWagner(_ParserScraper):
    url = 'http://www.hs.fi/viivijawagner/'
    imageSearch = '//meta[@property="og:image"]/@content'
    prevSearch = '//a[d:class("prev")]'
    latestSearch = '//div[d:class("cartoon-content")]//a'
    starter = indirectStarter
    lang = 'fi'

    def namer(self, image_url, page_url):
        return page_url.rsplit('-', 1)[1].split('.')[0]


class VirmirWorld(_ParserScraper):
    url = 'http://world.virmir.com/'
    stripUrl = url + 'comic.php?story=%s&page=%s'
    firstStripUrl = stripUrl % ('1', '1')
    imageSearch = '//div[@class="comic"]//img'
    prevSearch = '//a[contains(@class, "prev")]'

    def getIndexStripUrl(self, index):
        index = index.split('-')
        return self.stripUrl % (index[0], index[1])


class Vreakerz(ParserScraper):
    url = 'http://vreakerz.angrykitten.nl/'
    stripUrl = url + 'stories/read/%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src, "storypages")]'
    prevSearch = '//a[@class="btn-prior"]'



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2021 Tobias Gruetzmacher
import json
import multiprocessing
import os
import re
import warnings
from urllib.parse import urlsplit


# Dictionary with per-host locks.
_locks = {}
# Allowed number of connections per host
MaxConnections = 2
# Maximum number of strips to get to test a comic
MaxStrips = 5
# Match (already-escaped) archive.org URL
ARCHIVE_ORG_MATCH = re.compile(r'(?<=web\\.archive\\.org/web)/\d+/')
# Matches some (maybe-escaped - because Python 2) printf-style format specifiers
PRINTF_MATCH = re.compile(r'\\?%[0-9]*[sd]')
# Classes where the modules are very similar, so that testing the history of
# each modules doesn't make much sense
standarized_modules = {
    'ComicSherpa',
    'ComicsKingdom',
    'GoComics',
    'MangaDex',
    'WebToons',
}
# Already seen classes
seen_modules = set()


def get_lock(host):
    """Get bounded semphore for given host."""
    if host not in _locks:
        _locks[host] = multiprocessing.BoundedSemaphore(MaxConnections)
    return _locks[host]


def test_comicmodule(tmpdir, scraperobj, worker_id):
    '''Test a scraper. It must be able to traverse backward for at least 5
    strips from the start, and find strip images on at least 4 pages.'''
    # Limit number of connections to one host.
    host = urlsplit(scraperobj.url).hostname
    with get_lock(host):
        maxstrips = MaxStrips
        parts = scraperobj.name.split('/', maxsplit=1)
        if len(parts) > 1 and parts[0] in standarized_modules:
            if parts[0] in seen_modules:
                maxstrips = 1
            else:
                seen_modules.add(parts[0])

        _test_comic(str(tmpdir), scraperobj, maxstrips)


def _test_comic(outdir, scraperobj, maxstrips):
    num_strips = 0
    strip = None
    files = []
    PROXYMAP.apply(scraperobj.name)
    for strip in scraperobj.getStrips(maxstrips):
        files.append(_check_strip(outdir, strip,
                                  scraperobj.multipleImagesPerStrip))

        if num_strips > 0:
            _check_stripurl(strip, scraperobj)
        num_strips += 1

    if scraperobj.prevSearch and not scraperobj.hitFirstStripUrl:
        # subtract the number of skipped URLs with no image from the expected
        # image number
        num_strips_expected = maxstrips - len(scraperobj.skippedUrls)
        msg = 'Traversed %d strips instead of %d.' % (num_strips,
                                                      num_strips_expected)
        if strip:
            msg += " Check the prevSearch pattern at %s" % strip.strip_url
        assert num_strips == num_strips_expected, msg
        if strip:
            _check_scraperesult(files, num_strips_expected, strip, scraperobj)


def _check_strip(outdir, strip, multipleImagesPerStrip):
    '''Check that a specific page yields images and the comic module correctly
    declares if there are multiple images per page.'''
    images = []
    files = []
    for image in strip.getImages():
        images.append(image.url)

        # write a fake image (to download less)
        fakeimg = image._fnbase(outdir) + '.fake'
        with open(fakeimg, 'w') as f:
            f.write("fake image for testing")

        fn, _ = image.save(outdir)
        files.append(fn)
    assert images, 'failed to find images at %s' % strip.strip_url
    if not multipleImagesPerStrip:
        assert len(images) == 1, 'found more than 1 image at {}: {}'.format(
            strip.strip_url, images)
    return files


def _check_scraperesult(saved_images, num_images_expected, strip, scraperobj):
    '''Check that exactly or for multiple pages at least num_strips images are
    saved. This checks saved files, ie. it detects duplicate filenames.'''
    num_images = len(saved_images)

    attrs = (num_images, saved_images, num_images_expected)
    if scraperobj.multipleImagesPerStrip:
        err = 'saved %d %s instead of at least %d images' % attrs
        assert num_images >= num_images_expected, err
    else:
        err = 'saved %d %s instead of %d images' % attrs
        assert num_images == num_images_expected, err


def _check_stripurl(strip, scraperobj):
    if not scraperobj.stripUrl:
        # no indexing support
        return
    # test that the stripUrl regex matches the retrieved strip URL
    urlmatch = re.escape(scraperobj.stripUrl)
    urlmatch = PRINTF_MATCH.sub('.+', urlmatch)
    urlmatch = ARCHIVE_ORG_MATCH.sub(r'/\\d+/', urlmatch)
    ro = re.compile(urlmatch)
    mo = ro.match(strip.strip_url)
    if not mo:
        warnings.warn('strip URL {!r} does not match stripUrl pattern {}'.format(
            strip.strip_url, urlmatch))


class ProxyConfig:
    """Loads proxy config from an environment variable and applies it for each test."""
    def __init__(self):
        self.config = {}
        if 'PROXYMAP' in os.environ:
            for regex, server in json.loads(os.environ['PROXYMAP']).items():
                self.config[re.compile(regex)] = server

    def apply(self, name):
        useserver = ''
        for regex, server in self.config.items():
            if regex.match(name):
                useserver = server
                break
        os.environ['http_proxy'] = useserver
        os.environ['https_proxy'] = useserver


# External proxy config to fetch some modules via proxies
PROXYMAP = ProxyConfig()



#!/usr/bin/env python3
# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2020 Tobias Gruetzmacher
"""
Creates a CBZ file in the comic directory.
Uses an ordered symlink directory (see order-symlinks.py) if it exists,
else the plain files are used.
"""

import sys
import os
import zipfile

from dosagelib.configuration import App


# known image file extensions
ImageExts = (
    ".jpg",
    ".jpeg",
    ".gif",
    ".png",
)


def is_image(filename):
    """Determine if given filename is an image."""
    # note: isfile() also accepts symlinks
    return os.path.isfile(filename) and filename.lower().endswith(ImageExts)


def get_cbz_comment():
    """Return a UTF-8 encoded comment no longer than 65535 bytes.
    At the moment this just returns the application name and version,
    since cbz readers do not seem to use the comment string anyway."""
    return (u"Created by %s" % App).encode('utf-8')


def create_cbz(directory):
    """Creates or updates a CBZ from files in the given comic directory."""
    if not os.path.isdir(directory):
        print("ERROR: Directory", directory, "not found.")
        return
    base = os.path.basename(directory.rstrip(os.path.sep))
    zipname = '%s.cbz' % base
    zipname = os.path.join(directory, zipname)
    d = os.path.join(directory, 'inorder')
    if os.path.isdir(d):
        # use directory with ordered symlinks
        directory = d
    if os.path.exists(zipname):
        os.remove(zipname)
    with zipfile.ZipFile(zipname, 'w') as myzip:
        for filename in sorted(os.listdir(d)):
            fullname = os.path.join(d, filename)
            if is_image(fullname):
                myzip.write(fullname)
        myzip.comment = get_cbz_comment()
    print("INFO: Created", zipname)


if __name__ == '__main__':
    if len(sys.argv) > 1:
        for arg in sys.argv[1:]:
            create_cbz(arg)
    else:
        print("Usage: %s <comic-dir>..." % os.path.basename(sys.argv[0]))
